@@ -1,4 +1,4 @@
-# $Id: browser.tcl,v 1.45 2004/06/17 21:30:30 aleksey Exp $
+# $Id: browser.tcl,v 1.50 2005/08/06 22:13:32 aleksey Exp $
 
 option add *JBrowser.fill          Black	widgetDefault
 option add *JBrowser.nscolor       MidnightBlue widgetDefault
@@ -147,7 +147,7 @@ proc browser::recv {bw jid res child} {
 
 	    add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
 		-fill $config(fill)
-	    set browser(nchilds,$jid) 1
+	    set browser(nchilds,$bw,$jid) 1
 	} else {
 	    browser::process $bw $jid $child 0
 	}
@@ -198,8 +198,12 @@ proc browser::process {bw from item level} {
 	incr nchilds
     }
 
-    set browser(nchilds,$jid) $nchilds
-    set curchilds [$tw nodes [jid_to_tag $jid]]
+    set browser(nchilds,$bw,$jid) $nchilds
+    set node [jid_to_tag $jid]
+    if {![info exists browser(sort,$bw,$node)]} {
+	set browser(sort,$bw,$node) sort
+    }
+    set curchilds [$tw nodes $node]
 
     if {$level == 0} {
 	foreach c $curchilds {
@@ -207,23 +211,24 @@ proc browser::process {bw from item level} {
 		$tw delete $c
 	    }
 	}
+	browser::browser_action $browser(sort,$bw,$node) $bw $node
 	update idletasks
     }
     debugmsg browser [list $childs $curchilds]
 
-    return [jid_to_tag $jid]
+    return $node
 }
 
-set browsericon(group)   [Bitmap::get [pixmap browser group_on.gif]]
+set browsericon(group)   [Bitmap::get [pixmap browser conference.gif]]
 set browsericon(user)    [Bitmap::get [pixmap browser user.gif]]
 set browsericon(jud)     [Bitmap::get [pixmap browser jud.gif]]
-set browsericon(aim)     [Bitmap::get [pixmap browser aim_online.gif]]
-set browsericon(icq)     [Bitmap::get [pixmap browser icq_online.gif]]
-set browsericon(msn)     [Bitmap::get [pixmap browser msn_online.gif]]
-set browsericon(gg)      [Bitmap::get [pixmap browser gg_online.gif]]
-set browsericon(weather) [Bitmap::get [pixmap browser weather_online.gif]]
-set browsericon(yahoo)   [Bitmap::get [pixmap browser yahoo_online.gif]]
-set browsericon(message) [Bitmap::get [pixmap browser glade-message.gif]]
+set browsericon(aim)     [Bitmap::get [pixmap browser aim.gif]]
+set browsericon(icq)     [Bitmap::get [pixmap browser icq.gif]]
+set browsericon(msn)     [Bitmap::get [pixmap browser msn.gif]]
+set browsericon(gg)      [Bitmap::get [pixmap browser gg.gif]]
+set browsericon(weather) [Bitmap::get [pixmap browser weather.gif]]
+set browsericon(yahoo)   [Bitmap::get [pixmap browser yahoo.gif]]
+set browsericon(message) [Bitmap::get [pixmap browser message.gif]]
 image create photo ""
 
 proc browser::item_icon {category type} {
@@ -320,7 +325,7 @@ proc browser::item_desc {jid name} {
     }
 }
 
-proc browser::item_balloon_text {jid category type name version} {
+proc browser::item_balloon_text {bw jid category type name version} {
     variable browser
     set text "$jid: "
     set delim ""
@@ -335,7 +340,7 @@ proc browser::item_balloon_text {jid category type name version} {
     if {![cequal $version {}]} {
 	append text "$delim[::msgcat::mc Version:] $version"
     }
-    append text "\n[::msgcat::mc {Number of children:}] $browser(nchilds,$jid)"
+    append text "\n[::msgcat::mc {Number of children:}] $browser(nchilds,$bw,$jid)"
     return $text
 }
 
@@ -485,15 +490,18 @@ $m add command -label [::msgcat::mc "Join group..."] -command {
 	    -connection $::browser::browser(connid,$::browser::headwindow)
 }
 $m add command -label [::msgcat::mc "Add conference..."] -command {
-    set gra_group [node_from_jid $::browser::headjid]
-    set gra_server [server_from_jid $::browser::headjid]
-    add_group_dialog -connection $::browser::browser(connid,$::browser::headwindow)
+    plugins::conferences::add_conference_dialog \
+	-group [node_from_jid $::browser::headjid] \
+	-server [server_from_jid $::browser::headjid] \
+	-connection $::browser::browser(connid,$::browser::headwindow)
 }
 $m add separator
 $m add command -label [::msgcat::mc "Browse"] \
       -command {browser::browser_action browse $::browser::headwindow $::browser::headnode}
-$m add command -label [::msgcat::mc "Sort"] \
+$m add command -label [::msgcat::mc "Sort items by name"] \
       -command {browser::browser_action sort $::browser::headwindow $::browser::headnode}
+$m add command -label [::msgcat::mc "Sort items by JID"] \
+      -command {browser::browser_action sortjid $::browser::headwindow $::browser::headnode}
 
 if {[winfo exists [set m .b3popmenu]]} {
     destroy $m
@@ -501,8 +509,10 @@ if {[winfo exists [set m .b3popmenu]]} {
 menu $m -tearoff 0
 $m add command -label [::msgcat::mc "Browse"] \
       -command {browser::browser_action browse $::browser::headwindow $::browser::headnode}
-$m add command -label [::msgcat::mc "Sort"] \
+$m add command -label [::msgcat::mc "Sort items by name"] \
       -command {browser::browser_action sort $::browser::headwindow $::browser::headnode}
+$m add command -label [::msgcat::mc "Sort items by JID"] \
+      -command {browser::browser_action sortjid $::browser::headwindow $::browser::headnode}
 
 if {[winfo exists [set m .b4popmenu]]} {
     destroy $m
@@ -569,13 +579,50 @@ proc browser::browser_action {action bw node} {
 	}
 
 	jid/sort {
+	    set browser(sort,$bw,$node) sort
+	    set namespaces {}
             set children {}
             foreach child [$tw nodes $node] {
 		set data [$tw itemcget $child -data]
-                lappend children [list $child [lindex $data 4]]
+		switch -- [lindex $data 0] {
+		    ns {
+			lappend namespaces [list $child [lindex $data 4]]
+		    }
+		    default {
+			lappend children [list $child [lindex $data 4]]
+		    }
+		}
             }
             set neworder {}
-            foreach child [lsort -index 1 $children] {
+            foreach child [concat $namespaces \
+				  [lsort -dictionary -index 1 $children]] {
+                lappend neworder [lindex $child 0]
+            }
+            $tw reorder $node $neworder
+
+            foreach child [$tw nodes $node] {
+                browser::browser_action $action $bw $child
+            }
+	}
+
+	jid/sortjid {
+	    set browser(sort,$bw,$node) sortjid
+	    set namespaces {}
+            set children {}
+            foreach child [$tw nodes $node] {
+		set data [$tw itemcget $child -data]
+		switch -- [lindex $data 0] {
+		    ns {
+			lappend namespaces [list $child [lindex $data 1]]
+		    }
+		    default {
+			lappend children [list $child [lindex $data 1]]
+		    }
+		}
+            }
+            set neworder {}
+            foreach child [concat $namespaces \
+				  [lsort -dictionary -index 1 $children]] {
                 lappend neworder [lindex $child 0]
             }
             $tw reorder $node $neworder
@@ -588,7 +635,6 @@ proc browser::browser_action {action bw node} {
 	default {
 	}
     }
-
 }
 
 proc browser::textballoon {bw action X Y node} {
@@ -606,11 +652,11 @@ proc browser::textballoon {bw action X Y node} {
     switch -- $action {
 	enter {
 	    balloon::set_text \
-		[item_balloon_text $jid $category $type $name $version]
+		[item_balloon_text $bw $jid $category $type $name $version]
 	}
 	motion {
 	    balloon::on_mouse_move \
-		[item_balloon_text $jid $category $type $name $version] \
+		[item_balloon_text $bw $jid $category $type $name $version] \
 		$X $Y
 	}
 	leave {
