@@ -1,4 +1,4 @@
-# $Id: chats.tcl,v 1.125 2004/07/10 20:05:04 aleksey Exp $
+# $Id: chats.tcl,v 1.138 2005/08/14 20:44:10 aleksey Exp $
 
 package require textutil
 
@@ -19,8 +19,11 @@ namespace eval chat {
     set enrichid 0
     custom::defgroup Chat [::msgcat::mc "Chat options."] -group Tkabber
     custom::defvar options(smart_scroll) 0 \
-	    [::msgcat::mc "Enable chat window autoscroll only when last message is shown."] -type boolean -group Chat
-    custom::defvar options(stop_scroll) 0 [::msgcat::mc "Stop chat window autoscroll."] -type boolean -group Chat
+	[::msgcat::mc "Enable chat window autoscroll only when last message is shown."] \
+	-type boolean -group Chat
+    custom::defvar options(stop_scroll) 0 \
+	[::msgcat::mc "Stop chat window autoscroll."] \
+	-type boolean -group Chat
     custom::defvar options(emphasize) 1 \
 	[::msgcat::mc "Enable messages emphasize."] \
 	-type boolean -group Chat \
@@ -33,7 +36,8 @@ namespace eval chat {
 	-type radio -group Chat \
 	-values [list normal [::msgcat::mc "Normal"] \
 		      chat [string trim [::msgcat::mc "Chat "]]]
-    set url_regexp {^(http://|https://|www\.|ftp://|ftp\.)}
+    set url_regexp \
+	{^([^\w\d]*)((http://|https://|www\.|ftp://|ftp\.).*[\w\d/])([^\w\d]*)$}
 }
 
 set chat_width 50
@@ -41,33 +45,27 @@ set chat_height 18
 
 plugins::load [file join plugins chat]
 
-proc chat::get_nick {jid type} {
+proc chat::get_nick {connid jid type} {
     variable chats
 
     set nick $jid
     switch -- $type {
 	chat {
 	    set group [node_and_server_from_jid $jid]
-	    foreach connid [jlib::connections] {
-		set chatid [chatid $connid $group]
-		if {[is_groupchat $chatid]} {
-		    return [resource_from_jid $jid]
-		} else {
-		    set nick [roster::itemconfig $connid \
-				  [roster::find_jid $connid $jid] -name]
-		    if {$nick != ""} {
-			return $nick
-		    }
+	    set chatid1 [chatid $connid $group]
+	    if {[is_groupchat $chatid1]} {
+		set nick [resource_from_jid $jid]
+	    } else {
+		set nick [roster::itemconfig $connid \
+			      [roster::find_jid $connid $jid] -name]
+		if {$nick == ""} {
+		    set nick [node_from_jid $jid]
 		}
 	    }
-	    set nick [node_from_jid $jid]
 	}
 	groupchat {
-	    set group [node_and_server_from_jid $jid]
 	    set nick [resource_from_jid $jid]
 	}
-	default {
-	}
     }
     return $nick
 }
@@ -91,8 +89,8 @@ proc chat::get_jid {chatid} {
     return [lindex $chatid 1]
 }
 
-proc client:message {connid from id type subject body err thread priority x} {
-    debugmsg chat "MESSAGE: $connid; $from; $id $type; $subject; $body; $err;\
+proc client:message {connid from id type is_subject subject body err thread priority x} {
+    debugmsg chat "MESSAGE: $connid; $from; $id $type; $is_subject; $subject; $body; $err;\
 $thread; $priority; $x"
 
     if {![cequal [info commands ::ssj::encrypted:input] ""]} {
@@ -107,7 +105,7 @@ $thread; $priority; $x"
 		continue
 	    } elseif {[cequal $chdata ""]} {
 # in case the sender didn't check the exit code from gpg...
-	    } elseif {[catch { ssj::encrypted:input $from $chdata } body]} {
+	    } elseif {[catch { ssj::encrypted:input $connid $from $chdata } body]} {
 		set body [format [::msgcat::mc ">>> Unable to decipher data: %s <<<"] $body]
 	    } else {
 		lappend xs $xe
@@ -118,10 +116,10 @@ $thread; $priority; $x"
     }
 
     chat::process_message \
-	$connid $from $id $type $subject $body $err $thread $priority $x
+	$connid $from $id $type $is_subject $subject $body $err $thread $priority $x
 }
 
-proc chat::process_message {connid from id type subject body err thread priority x} {
+proc chat::process_message {connid from id type is_subject subject body err thread priority x} {
     variable options
 
     if {$type == ""} {
@@ -129,14 +127,14 @@ proc chat::process_message {connid from id type subject body err thread priority
     }
 
     set from [tolower_node_and_domain $from]
-    hook::run examine_message_hook $from $id $type $subject $body $err \
+    hook::run examine_message_hook $from $id $type $is_subject $subject $body $err \
 				   $thread $priority $x
 
     hook::run process_message_hook \
-	$connid $from $id $type $subject $body $err $thread $priority $x
+	$connid $from $id $type $is_subject $subject $body $err $thread $priority $x
 }
 
-proc chat::process_message_fallback {connid from id type subject body err thread priority x} {
+proc chat::process_message_fallback {connid from id type is_subject subject body err thread priority x} {
     global font
     variable opened
     variable chats
@@ -165,7 +163,7 @@ proc chat::process_message_fallback {connid from id type subject body err thread
 
 	    if {![is_groupchat $chatid]} return
 
-	    if {![cequal $subject ""]} {
+	    if {$is_subject} {
 		set_subject $chatid $subject
 		if {[cequal $body ""]} {
 		    set body [format [::msgcat::mc "/me has changed the subject to: %s"] $subject]
@@ -173,7 +171,7 @@ proc chat::process_message_fallback {connid from id type subject body err thread
 	    }
 	}
 	error {
-	    if {![cequal $subject ""]} {
+	    if {$is_subject} {
 		set body "subject: "
 		restore_subject $chatid
 	    } else {
@@ -213,7 +211,7 @@ proc chat::window_titles {chatid} {
 	if {[is_groupchat $chatid]} {
 	    set tabtitlename [node_from_jid $jid]
 	} else {
-	    set tabtitlename [get_nick $jid chat]
+	    set tabtitlename [get_nick $connid $jid chat]
 	}
 	if {$tabtitlename == ""} {
 	    set tabtitlename $titlename
@@ -229,13 +227,20 @@ proc chat::reconnect_groupchats {connid} {
     if {![info exists chats(opened)]} return
 
     foreach chatid $chats(opened) {
-	if {[cequal [get_connid $chatid] $connid] && \
-		[is_groupchat $chatid]} {
-	    set grouproster(users,$chatid) {}
-
-	    join_group [get_jid $chatid] \
-		-nick [get_our_groupchat_nick $chatid] \
-		-connection $connid
+	if {[cequal [get_connid $chatid] $connid]} {
+	    if {[is_groupchat $chatid]} {
+		if {[info exists ::muc::muc_password($chatid)]} {
+		    set password $::muc::muc_password($chatid)
+		} else {
+		    set password ""
+		}
+		join_group [get_jid $chatid] \
+		    -nick [get_our_groupchat_nick $chatid] \
+		    -password $password \
+		    -connection $connid
+	    } else {
+		set chats(status,$chatid) connected
+	    }
 	}
     }
 }
@@ -250,7 +255,9 @@ proc chat::disconnect_groupchats {connid} {
     foreach chatid $chats(opened) {
 	if {[cequal $connid {}] || [cequal [get_connid $chatid] $connid]} {
 	    set chatw [chat_win $chatid]
-	    if {[winfo exists $chatw]} {
+	    if {[winfo exists $chatw] && \
+		    $chats(status,$chatid) != "disconnected"} {
+		set chats(status,$chatid) disconnected
 		add_message $chatid [get_jid $chatid] error \
 		    [::msgcat::mc "Disconnected"] {}
 	    }
@@ -267,6 +274,8 @@ proc chat::open_window {chatid type} {
     variable chats
     variable chat_id
     variable options
+    global grouproster
+    global statusdesc
 
     set connid [get_connid $chatid]
     set jid [get_jid $chatid]
@@ -276,6 +285,10 @@ proc chat::open_window {chatid type} {
     set cw [winid $chatid]
 
     if {[winfo exists $cw]} {
+	if {$type == "groupchat" && $chats(status,$chatid) == "disconnected"} {
+	    set grouproster(users,$chatid) {}
+	}
+
 	if {!$::usetabbar && \
 		[info exists ::raise_on_activity] && $::raise_on_activity} {
 	    if {$type == "chat"} {
@@ -290,6 +303,12 @@ proc chat::open_window {chatid type} {
 
     set chats(type,$chatid) $type
     set chats(subject,$chatid) ""
+    if {$type == "groupchat"} {
+	set chats(status,$chatid) disconnected
+    } else {
+	set chats(status,$chatid) connected
+    }
+    set chats(exit_status,$chatid) ""
     debugmsg chat "OURJID: [our_jid $chatid]"
     lappend chats(opened) $chatid
     
@@ -313,26 +332,29 @@ proc chat::open_window {chatid type} {
     pack $cw.status -side top -fill x
     if {[cequal $type chat]} {
 	set status [get_user_status $connid $jid]
-	label $cw.status.icon -image $ifacetk::roster::rostericon(user,$status)
+	Label $cw.status.icon \
+	    -image [ifacetk::roster::get_jid_icon $connid $jid] \
+	    -helptext [get_user_status_desc $connid $jid]
 	pack $cw.status.icon -side left
 	if {$options(display_status_description)} {
-	    Label $cw.status.desc -text "($status)" -font $font \
+	    Label $cw.status.desc -text "($statusdesc($status))" -font $font \
 		-helptext [get_user_status_desc $connid $jid]
 	    pack $cw.status.desc -side left
 	}
     }
     if {![cequal [info commands ::ssj::encrypt:toggleP] ""]} {
         if {[cequal [set sjid [roster::find_jid $connid $jid]] ""]} {
-            set sjid $chatid
+            set sjid $jid
         }
         Button $cw.status.encrypted -relief flat \
-            -image [message::encrypted:icon $sjid] \
+            -image [message::encrypted:icon $connid $sjid] \
             -helptype balloon \
             -helptext [::msgcat::mc "Toggle encryption"] \
-            -command [list ssj::encrypt:toggleP $sjid]
+            -command [list ssj::encrypt:toggleP $connid $sjid]
         ssj::encrypted:trace "
-            $cw.status.encrypted configure -image \[message::encrypted:icon $sjid\]
-        " $sjid
+            $cw.status.encrypted configure \
+		-image \[message::encrypted:icon $connid $sjid\]
+        " $connid $sjid
         pack $cw.status.encrypted -side left
     }
     if {[cequal $type chat]} {
@@ -352,6 +374,8 @@ proc chat::open_window {chatid type} {
 
 	bind $cw.status.subject <Return> \
 	    [list chat::change_subject [double% $chatid]]
+	bind $cw.status.subject <Escape> \
+	    [list chat::restore_subject [double% $chatid]]
     }
     foreach tag [bind Menubutton] {
         if {[string first 1 $tag] >= 0} {
@@ -363,6 +387,7 @@ proc chat::open_window {chatid type} {
     PanedWin $cw.pw0 -side right -pad 0 -width 8
     pack $cw.pw0 -fill both -expand yes
 
+
     set upw [$cw.pw0 add -weight 1 -minsize 0]
     set dow [$cw.pw0 add -weight 0 -minsize 0]
 
@@ -393,14 +418,8 @@ proc chat::open_window {chatid type} {
 	pack $uw.users -fill both -side right -expand yes
 	[winfo parent $uw] configure -width $rosterwidth
 	
-	global grouproster
-	
 	set grouproster(users,$chatid) {}
 
-	#roster::addline $uw.users group Users users
-
-	client:presence $connid $jid "" "" {}
-
 	set pack_in $cf
     } else {
 	set cf $cw
@@ -410,7 +429,7 @@ proc chat::open_window {chatid type} {
     set chats(chatwin,$chatid) $cf.chat
 
     set csw [ScrolledWindow $cf.csw -scrollbar vertical -auto none]
-    pack $csw -expand yes -fill both -side left -in $pack_in
+    pack $csw -expand yes -fill both -side top -in $pack_in
     text $cf.chat -width $chat_width -height $chat_height -font $font -wrap word
     $csw setwidget $cf.chat
 
@@ -433,8 +452,13 @@ proc chat::open_window {chatid type} {
     $cf.chat tag configure italic -font $font_italic
     $cf.chat tag configure bold_italic -font $font_bold_italic
     $cf.chat tag configure underlined -underline 1
-    $cf.chat tag configure emphasized -elide 0
-    $cf.chat tag configure nonemphasized -elide 1
+    if {$options(emphasize)} {
+	$cf.chat tag configure emphasized -elide 0
+	$cf.chat tag configure nonemphasized -elide 1
+    } else {
+	$cf.chat tag configure emphasized -elide 1
+	$cf.chat tag configure nonemphasized -elide 0
+    }
 
 
     $cf.chat configure -state disabled
@@ -461,7 +485,6 @@ proc chat::open_window {chatid type} {
     hook::run open_chat_post_hook $chatid $type
 }
 
-
 proc chat::close_window {chatid} {
     variable opened
     variable chats
@@ -482,10 +505,15 @@ proc chat::close_window {chatid} {
     debugmsg chat $chats(opened)
 
     if {[is_groupchat $chatid]} {
-	jlib::send_presence \
-	    -to $jid/[get_our_groupchat_nick $chatid] \
-	    -connection $connid \
-	    -type unavailable
+	set command [list jlib::send_presence \
+			      -to $jid/[get_our_groupchat_nick $chatid] \
+			      -type unavailable \
+			      -connection $connid]
+	if {![cequal $chats(exit_status,$chatid) ""]} {
+	    lappend command -stat $chats(exit_status,$chatid)
+	}
+	eval $command
+	set chats(status,$chatid) disconnected
 	client:presence $connid $jid unavailable "" {}
 	foreach jid [get_jids_of_user $connid $jid] {
 	    client:presence $connid $jid unavailable "" {}
@@ -545,9 +573,9 @@ proc chat::create_user_menu {path chatid} {
 	-command [list ifacetk::roster::send_users_dialog $jid -connection $connid]
     ft::create_menu $m $jid -connection $connid
     $m add separator
-    $m add command -label [::msgcat::mc "Show info..."] \
+    $m add command -label [::msgcat::mc "Show info"] \
 	-command [list userinfo::open $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Show history..."] \
+    $m add command -label [::msgcat::mc "Show history"] \
 	-command [list logger::show_log $jid]
     return $path
 }
@@ -556,13 +584,24 @@ proc chat::create_conference_menu {path chatid} {
     set m [menu $path -tearoff 0]
     set connid [get_connid $chatid]
     set jid [get_jid $chatid]
-    # TODO: connid
     $m add command -label [::msgcat::mc "Invite users..."] \
-	-command [list chat::invite_dialog2 $jid]
-    $m add command -label [::msgcat::mc "Show history..."] \
+	-command [list chat::invite_dialog2 $jid 0 -connection $connid]
+
+    if {[info exists ::muc::muc_password($chatid)]} {
+	set password $::muc::muc_password($chatid)
+    } else {
+	set password ""
+    }
+    $m add command -label [::msgcat::mc "Add conference..."] \
+	-command [list plugins::conferences::add_conference_dialog \
+		       -group [node_from_jid $jid] \
+		       -server [server_from_jid $jid] \
+		       -password $password \
+		       -connection $connid]
+    $m add command -label [::msgcat::mc "Show history"] \
 	-command [list logger::show_log $jid]
     add_conference_custom_presence_menu $m $chatid
-    after idle [list hook::run chat_create_conference_menu_hook $path $chatid]
+    hook::run chat_create_conference_menu_hook $path $chatid
     return $path
 }
 
@@ -572,7 +611,7 @@ proc chat::add_conference_custom_presence_menu {m chatid} {
     set jid [get_jid $chatid]
     set connid [get_connid $chatid]
 
-    $mm add command -label [::msgcat::mc "Online"] \
+    $mm add command -label [::msgcat::mc "Available"] \
 	-command [list send_custom_presence $jid available \
 			   -type group -connection $connid]
     $mm add command -label [::msgcat::mc "Free to chat"] \
@@ -587,7 +626,7 @@ proc chat::add_conference_custom_presence_menu {m chatid} {
     $mm add command -label [::msgcat::mc "Do not disturb"] \
 	-command [list send_custom_presence $jid dnd \
 			   -type group -connection $connid]
-    #$mm add command -label [::msgcat::mc "Offline"] \
+    #$mm add command -label [::msgcat::mc "Unavailable"] \
     #    -command [list send_custom_presence $jid unavailable \
     #   		   -type group -connection $connid]
 
@@ -697,19 +736,19 @@ proc chat::add_emoteiconed_text {chatw body defaulttag} {
 		continue
 	    }
 	}
-	if {[regexp -- $url_regexp $word]} {
+	if {[regexp -- $url_regexp $word ignore head url ignore2 tail]} {
 	    variable urlid
 	    set tag url$urlid
 	    set urlfg    [option get $chatw urlforeground       Text]
 	    set urlactfg [option get $chatw urlactiveforeground Text]
 	    $chatw tag configure $tag -foreground $urlfg -underline 1
-	    $chatw tag bind $tag <1> [list browseurl [double% $word]]
+	    $chatw tag bind $tag <1> [list browseurl [double% $url]]
 	    $chatw tag bind $tag <Any-Enter> \
 		[list chat::highlighttext $chatw $tag $urlactfg hand2]
 	    $chatw tag bind $tag <Any-Leave> \
 		[list chat::highlighttext $chatw $tag $urlfg xterm]
 	    
-	    $chatw insert end $word $tag
+	    $chatw insert end $head $defaulttag $url $tag $tail $defaulttag
 	    incr urlid
 	} else {
 	    if {0 && !$options(emphasize)} {
@@ -820,10 +859,11 @@ proc chat::change_presence {connid jid status} {
     global grouproster
     variable chats
     variable opened
+    global statusdesc
 
     set group [node_and_server_from_jid $jid]
     set chatid [chatid $connid $group]
-    set nick [get_nick $jid groupchat]
+    set nick [get_nick $connid $jid groupchat]
 
     if {[info exists opened($chatid)]} {
 	if {[is_groupchat $chatid]} {
@@ -836,12 +876,16 @@ proc chat::change_presence {connid jid status} {
 		    add_message $chatid $group error \
 			[format [::msgcat::mc "Error %s"] \
 			     [get_jid_presence_info status $connid $jid]] {}
+			set chats(status,$chatid) disconnected
+			client:presence $connid $group unavailable "" {}
 		}
 		if {$status == "unavailable" && [is_our_jid $chatid $jid]} {
 		    if {[muc::is_compatible $group] || \
 			    ![muc::is_changing_nick $chatid]} {
 			add_message $chatid $group error \
 			    [::msgcat::mc "Disconnected"] {}
+			set chats(status,$chatid) disconnected
+			client:presence $connid $group unavailable "" {}
 		    }
 		}
 		debugmsg chat "$jid UNAVAILABLE"
@@ -852,6 +896,11 @@ proc chat::change_presence {connid jid status} {
 		debugmsg chat "GR: $grouproster(users,$chatid)"
 		chat::redraw_roster_after_idle $chatid
 	    } else {
+		if {$chats(status,$chatid) == "disconnected" && \
+			[is_our_jid $chatid $jid]} {
+		    set chats(status,$chatid) connected
+		    client:presence $connid $group "" "" {}
+		}
 		set userswin [users_win $chatid]
 		if {![lcontain $grouproster(users,$chatid) $jid]} {
 		    #roster::addline $userswin jid $nick $jid
@@ -875,11 +924,12 @@ proc chat::change_presence {connid jid status} {
     set cw [winid [chatid $connid $jid]]
 
     if {[winfo exists $cw.status.icon]} {
-	$cw.status.icon configure -image $::ifacetk::roster::rostericon(user,$status)
+	$cw.status.icon configure -image $::ifacetk::roster::rostericon(user,$status) \
+	    -helptext [get_user_status_desc $connid $jid]
     }
 
     if {[winfo exists $cw.status.desc]} {
-	$cw.status.desc configure -text "([get_user_status $connid $jid])" \
+	$cw.status.desc configure -text "($statusdesc([get_user_status $connid $jid]))" \
 	    -helptext [get_user_status_desc $connid $jid]
     }
 
@@ -950,7 +1000,7 @@ proc chat::redraw_roster {group} {
 	    [crange $level 1 end] [crange $level 1 end] 0
 	set jid_nicks {}
 	foreach jid $levelusers($level) {
-	    lappend jid_nicks [list $jid [get_nick $jid groupchat]]
+	    lappend jid_nicks [list $jid [get_nick $connid $jid groupchat]]
 	}
 	set jid_nicks [lsort -index 1 -dictionary $jid_nicks]
 
@@ -1076,7 +1126,8 @@ proc chat::invite_dialog {user {ignore_muc 0} args} {
     if {[catch { set nick [roster::get_label $user] }]} {
 	if {[catch {set nick [roster::get_label \
 				  [node_and_server_from_jid $user]] }]} {
-	    if {[catch { set nick [chat::get_nick $user groupchat] }]} {
+	    if {[catch { set nick [chat::get_nick $connid \
+						  $user groupchat] }]} {
 		set nick $user
 	    }
 	}
@@ -1085,14 +1136,17 @@ proc chat::invite_dialog {user {ignore_muc 0} args} {
     set titles {}
     set jids {}
     foreach chatid [lsort [array names opened]] {
-        if {[is_groupchat $chatid]} {
+        if {([get_connid $chatid] == $connid) && [is_groupchat $chatid]} {
 	    lappend jids $chatid [get_jid $chatid]
             lappend titles $chatid [node_from_jid [get_jid $chatid]]
 	}
     }
     if {[llength $titles] == 0} {
         MessageDlg ${gw}_err -aspect 50000 -icon info \
-	    -message [::msgcat::mc "No conferences in progress..."] -type user \
+	    -message \
+		[format [::msgcat::mc "No conferences for %s in progress..."] \
+		    [jlib::connection_jid $connid]] \
+	    -type user \
 	    -buttons ok -default 0 -cancel 0
         return
     }
@@ -1104,29 +1158,39 @@ proc chat::invite_dialog {user {ignore_muc 0} args} {
 	invite_gc $titles $jids
 }
 
-proc chat::invite_dialog2 {jid {ignore_muc 0}} {
+proc chat::invite_dialog2 {jid ignore_muc args} {
     variable chats
     variable opened
     global invite_gc
 
+    foreach {opt val} $args {
+	switch -- $opt {
+	    -connection { set connid $val }
+	}
+    }
+    if {![info exists connid]} {
+	set connid [jlib::route $jid]
+    }
+
     set gw .invite
     catch { destroy $gw }
 
     set title [node_from_jid $jid]
 
-    set choices {} 
-    set balloons {} 
-    foreach c [jlib::connections] { 
-	foreach choice [roster::get_jids $c] { 
-	    if {![cequal [roster::itemconfig $c $choice -category] conference]} {
-		lappend choices [list $c $choice] [roster::get_label $c $choice]
-		lappend balloons [list $c $choice] $choice 
-	    } 
+    set choices {}
+    set balloons {}
+    foreach choice [roster::get_jids $connid] {
+	if {![cequal [roster::itemconfig $connid $choice -category] conference]} {
+	    lappend choices [list $connid $choice] [roster::get_label $connid $choice]
+	    lappend balloons [list $connid $choice] $choice
 	} 
     } 
     if {[llength $choices] == 0} {
         MessageDlg ${gw}_err -aspect 50000 -icon info \
-	    -message [::msgcat::mc "No users in roster..."] -type user \
+	    -message \
+		[format [::msgcat::mc "No users in %s roster..."] \
+		     [jlib::connection_jid $connid]] \
+	    -type user \
 	    -buttons ok -default 0 -cancel 0
 	return
     }
@@ -1146,25 +1210,26 @@ proc chat::invitation {jid usersP ignore_muc {reason ""}} {
 	    lassign $choice con gc
 	    if {$usersP} {
 		set to $gc
-		set chatid $jid
+		set group $jid
 	    } else {            
 		set to $jid
-		set chatid $gc
+		set group $gc
 	    }
 	    if {[cequal $reason ""]} {
-		set reas [format [::msgcat::mc "Please join %s"] $chatid]
+		set reas [format [::msgcat::mc "Please join %s"] $group]
 	    } else {
 		set reas $reason
 	    }
-	    if {!$ignore_muc && [muc::is_compatible [chat::get_jid $chatid]]} {
-		muc::invitation $chatid $to $reas
+	    if {!$ignore_muc && [muc::is_compatible $group]} {
+		muc::invitation [chatid $con $group] $to $reas
 	    } else {
 		message::send_msg $to -type normal \
 		    -subject "Invitation" \
 		    -body $reas \
 		    -xlist [list [jlib::wrapper:createtag x \
 				-vars [list xmlns jabber:x:conference \
-				jid $chatid]]]
+				jid $group]]] \
+		    -connection $con
 	    }
 	}
     }
