@@ -1,4 +1,4 @@
-# $Id: disco.tcl,v 1.31 2004/06/30 22:00:39 aleksey Exp $
+# $Id: disco.tcl,v 1.37 2005/08/06 22:13:32 aleksey Exp $
 
 set ::NS(disco_items) "http://jabber.org/protocol/disco#items"
 set ::NS(disco_info)  "http://jabber.org/protocol/disco#info"
@@ -126,11 +126,12 @@ proc disco::request_info {jid node args} {
 		    [info exists disco(info,identities,$jid,$node)]} {
 		set identities $disco(info,identities,$jid,$node)
 		set features   $disco(info,features,$jid,$node)
+		set extras     $disco(info,extras,$jid,$node)
 		if {$handler != ""} {
-		    eval $handler [list OK $identities $features]
+		    eval $handler [list OK $identities $features $extras]
 		}
 		if {$cache != "first"} {
-		    return [list OK $identities $features]
+		    return [list OK $identities $features $extras]
 		}
 	    } elseif {$cache == "only"} {
 		return NO
@@ -156,19 +157,20 @@ proc disco::parse_info {jid node handler res child} {
 
     if {![cequal $res OK]} {
 	if {$handler != ""} {
-	    eval $handler [list ERR $child {}]
+	    eval $handler [list ERR $child {} {}]
 	}
-	hook::run disco_info_hook $jid $node ERR $child {}
+	hook::run disco_info_hook $jid $node ERR $child {} {}
 	return
     }
 
     set identities {}
     set features {}
+    set extras {}
 
-    jlib::wrapper:splitxml $child tag vars isempty chdata childrens
+    jlib::wrapper:splitxml $child tag vars isempty chdata children
 
-    foreach ch $childrens {
-	jlib::wrapper:splitxml $ch tag1 vars1 isempty1 chdata1 childrens1
+    foreach ch $children {
+	jlib::wrapper:splitxml $ch tag1 vars1 isempty1 chdata1 children1
 	switch -- $tag1 {
 	    identity {
 		lappend identities \
@@ -184,19 +186,27 @@ proc disco::parse_info {jid node handler res child} {
 		}
 		lappend features [list var $var]
 	    }
+	    default {
+		if {[jlib::wrapper:getattr $vars1 xmlns] == "jabber:x:data" && \
+			[jlib::wrapper:getattr $vars1 type] == "result"} {
+		    set extras [data::parse_xdata_results $children1]
+		}
+	    }
 	}
     }
 
     set disco(info,identities,$jid,$node) $identities
     set disco(info,features,$jid,$node) $features
+    set disco(info,extras,$jid,$node) $extras
 
     debugmsg disco \
-	"INFO: IDENTITIES [list $identities] FEATURES [list $features]"
+	"INFO: IDENTITIES [list $identities] FEATURES [list $features]\
+EXTRAS [list $extras]"
 
     if {$handler != ""} {
-	eval $handler [list OK $identities $features]
+	eval $handler [list OK $identities $features $extras]
     }
-    hook::run disco_info_hook $jid $node OK $identities $features
+    hook::run disco_info_hook $jid $node OK $identities $features $extras
 }
 
 proc disco::get_jid_name {jid node} {
@@ -208,6 +218,15 @@ proc disco::get_jid_name {jid node} {
     }
 }
 
+proc disco::get_jid_identities {jid node} {
+    variable disco
+    if {[info exists disco(info,identities,$jid,$node)]} {
+	return $disco(info,identities,$jid,$node)
+    } else {
+	return {}
+    }
+}
+
 ###############################################################################
 
 proc disco::info_query_get_handler {connid from child} {
@@ -320,16 +339,16 @@ iq::register_handler get query $::NS(disco_items) \
 namespace eval disco::browser {
     set winid 0
 
-    set icon(group)   [Bitmap::get [pixmap browser group_on.gif]]
+    set icon(group)   [Bitmap::get [pixmap browser conference.gif]]
     set icon(client)  [Bitmap::get [pixmap browser user.gif]]
     set icon(jud)     [Bitmap::get [pixmap browser jud.gif]]
-    set icon(aim)     [Bitmap::get [pixmap browser aim_online.gif]]
-    set icon(icq)     [Bitmap::get [pixmap browser icq_online.gif]]
-    set icon(msn)     [Bitmap::get [pixmap browser msn_online.gif]]
-    set icon(gg)      [Bitmap::get [pixmap browser gg_online.gif]]
-    set icon(weather) [Bitmap::get [pixmap browser weather_online.gif]]
-    set icon(yahoo)   [Bitmap::get [pixmap browser yahoo_online.gif]]
-    set icon(message) [Bitmap::get [pixmap browser glade-message.gif]]
+    set icon(aim)     [Bitmap::get [pixmap browser aim.gif]]
+    set icon(icq)     [Bitmap::get [pixmap browser icq.gif]]
+    set icon(msn)     [Bitmap::get [pixmap browser msn.gif]]
+    set icon(gg)      [Bitmap::get [pixmap browser gg.gif]]
+    set icon(weather) [Bitmap::get [pixmap browser weather.gif]]
+    set icon(yahoo)   [Bitmap::get [pixmap browser yahoo.gif]]
+    set icon(message) [Bitmap::get [pixmap browser message.gif]]
     image create photo ""
 
     custom::defvar disco_list {} [::msgcat::mc "List of discovered JIDs."] \
@@ -369,7 +388,7 @@ proc disco::browser::open_win {jid args} {
     incr winid
     set browser(connid,$w) $connid
 
-    add_win $w -title [::msgcat::mc "Jabber Discovery"] \
+    add_win $w -title [::msgcat::mc "Service Discovery"] \
 	-tabtitle [::msgcat::mc "Discovery"] \
 	-raisecmd [list focus $w.tree] \
 	-class JDisco
@@ -416,6 +435,8 @@ proc disco::browser::open_win {jid args} {
     set disco(tree,$w) $tw
     $tw bindText <Double-ButtonPress-1> \
 	[list [namespace current]::textaction $w]
+    $tw bindText <ButtonPress-3> \
+	[list [namespace current]::textpopup $w]
     $tw bindText <Any-Enter>  \
 	[list [namespace current]::textballoon $w enter  %X %Y]
     $tw bindText <Any-Motion> \
@@ -466,14 +487,14 @@ proc disco::browser::go {bw} {
     }
 }
 
-proc disco::browser::info_receive {jid node res identities features} {
+proc disco::browser::info_receive {jid node res identities features extras} {
     variable browser
 
     if {![info exists browser(opened)]} return
 
     foreach w $browser(opened) {
 	if {[winfo exists $w] && [lcontain $browser(required,$w) $jid]} {
-	    draw_info $w $jid $node $res $identities $features
+	    draw_info $w $jid $node $res $identities $features $extras
 	}
     }
 }
@@ -481,7 +502,7 @@ proc disco::browser::info_receive {jid node res identities features} {
 hook::add disco_info_hook \
     [namespace current]::disco::browser::info_receive
 
-proc disco::browser::draw_info {w jid node res identities features} {
+proc disco::browser::draw_info {w jid node res identities features extras} {
     variable browser
     variable config
     global font
@@ -534,6 +555,20 @@ proc disco::browser::draw_info {w jid node res identities features} {
 	    -fill $config(identitycolor)
     }
 
+    set extranodes {}
+    
+    foreach extra $extras {
+	lassign $extra var label value
+	set tnode [jid_to_tag "extra $var $jid $node"]
+	lappend extranodes $tnode
+	set data [list extra $var $jid $node]
+	set desc "$label ($var): $value"
+	set icon ""
+	
+	add_line $tw $parent_tag $tnode $icon $desc $data -font $font \
+	    -fill $config(identitycolor)
+    }
+
     set featurenodes {}
 
     foreach feature $features {
@@ -559,6 +594,7 @@ proc disco::browser::draw_info {w jid node res identities features} {
     }
 
     remove_old $tw $parent_tag identity $identitynodes
+    remove_old $tw $parent_tag extra    $extranodes
     remove_old $tw $parent_tag feature  $featurenodes
     remove_old $tw $parent_tag error_info {}
     reorder_node $tw $parent_tag
@@ -588,6 +624,9 @@ proc disco::browser::draw_items {w jid node res items} {
 
     set tnode [jid_to_tag [list $jid $node]]
     set parent_tag [jid_to_tag [list $jid $node]]
+    if {![info exists browser(sort,$w,$parent_tag)]} {
+	set browser(sort,$w,$parent_tag) sort
+    }
     set data [list item $jid $node]
     set desc $jid
     set icon ""
@@ -628,7 +667,8 @@ proc disco::browser::draw_items {w jid node res items} {
     }
     remove_old $tw $parent_tag item $itemnodes
     remove_old $tw $parent_tag error_items {}
-    reorder_node $tw $parent_tag
+    #reorder_node $tw $parent_tag
+    browser_action $browser(sort,$w,$parent_tag) $w $parent_tag
 }
 
 proc disco::browser::negotiate_feature {tw connid jid parent type} {
@@ -698,7 +738,7 @@ proc disco::browser::add_line {tw parent node icon desc data args} {
 }
 
 
-proc disco::browser::reorder_node {tw node} {
+proc disco::browser::reorder_node {tw node {order {}}} {
     set subnodes [$tw nodes $node]
 
     set identities {}
@@ -714,7 +754,11 @@ proc disco::browser::reorder_node {tw node} {
 	    feature  {lappend features   $sn}
 	}
     }
-    $tw reorder $node [concat $identities $features $items]
+    if {$order == {}} {
+	$tw reorder $node [concat $identities $features $items]
+    } else {
+	$tw reorder $node [concat $identities $features $order]
+    }
 }
 
 proc disco::browser::remove_old {tw node kind newnodes} {
@@ -827,6 +871,145 @@ proc disco::browser::textaction {bw tnode} {
     }
 }
 
+proc disco::browser::textpopup {bw tnode} {
+    variable browser
+
+    if {[winfo exists [set m .discopopupmenu]]} {
+	destroy $m
+    }
+    menu $m -tearoff 0
+
+    set tw $browser(tree,$bw)
+    set data [$tw itemcget $tnode -data]
+    set data2 [lassign $data type]
+
+    switch -- $type {
+	identity {
+	    return
+	}
+
+	feature {
+	    $m add command -label [::msgcat::mc "Browse"] \
+		-command [list [namespace current]::browser_action browse $bw $tnode]
+	}
+
+	item {
+	    lassign $data2 jid node
+	    set identities [[namespace parent]::get_jid_identities $jid $node]
+	    # TODO: use all identities
+	    set category [jlib::wrapper:getattr [lindex $identities 0] category]
+	    switch -- $category {
+		client {
+		    message::subject_menu $m \
+			   [lindex $data 1] message -connection $browser(connid,$bw)
+		    tk_popup $m [winfo pointerx .] [winfo pointery .]
+		    return
+		}
+
+		conference {
+		    if {[string first @ [set headjid [lindex $data 1]]] > 0} {
+			$m add command -label [::msgcat::mc "Join group..."] \
+			    -command [list join_group_dialog \
+					   -server [server_from_jid $headjid] \
+					   -group [node_from_jid $headjid] \
+					   -connection $browser(connid,$bw)]
+			$m add command -label [::msgcat::mc "Add conference..."] \
+			    -command [list plugins::conferences::add_conference_dialog \
+					   -group [node_from_jid $headjid] \
+					   -server [server_from_jid $headjid] \
+					   -connection $browser(connid,$bw)]
+			$m add separator
+		    }
+		}
+
+		default {
+		}
+	    }
+	    $m add command -label [::msgcat::mc "Browse"] \
+		-command [list [namespace current]::browser_action browse $bw $tnode]
+	    $m add command -label [::msgcat::mc "Sort items by name"] \
+		-command [list [namespace current]::browser_action sort $bw $tnode]
+	    $m add command -label [::msgcat::mc "Sort items by JID/node"] \
+		-command [list [namespace current]::browser_action sortjid $bw $tnode]
+	}
+    }
+
+    tk_popup $m [winfo pointerx .] [winfo pointery .]
+}
+
+proc disco::browser::browser_action {action bw tnode} {
+    variable browser
+
+    set tw $browser(tree,$bw)
+    set data [$tw itemcget $tnode -data]
+    set data2 [lassign $data type]
+
+    switch -glob -- $type/$action {
+	item/browse -
+	feature/browse {
+	    textaction $bw $tnode
+	}
+
+	item/sort {
+	    set browser(sort,$bw,$tnode) sort
+            set items {}
+            foreach child [$tw nodes $tnode] {
+		set data [lassign [$tw itemcget $child -data] type]
+		switch -- $type {
+		    item {
+			lassign $data jid node
+			lappend items \
+			    [list $child \
+				  [[namespace parent]::get_jid_name $jid $node]]
+		    }
+		}
+            }
+            set neworder {}
+            foreach item [lsort -dictionary -index 1 $items] {
+                lappend neworder [lindex $item 0]
+            }
+            reorder_node $tw $tnode $neworder
+
+            foreach child [$tw nodes $tnode] {
+                browser::browser_action $action $bw $child
+            }
+	}
+
+	item/sortjid {
+	    set browser(sort,$bw,$tnode) sortjid
+            set items {}
+	    set items_with_nodes {}
+            foreach child [$tw nodes $tnode] {
+		set data [lassign [$tw itemcget $child -data] type]
+		switch -- $type {
+		    item {
+			lassign $data jid node
+			if {$node != {}} {
+			    lappend items_with_nodes \
+				[list $child [join $data "\u0000"]]
+			} else {
+			    lappend items [list $child $jid]
+			}
+		    }
+		}
+            }
+            set neworder {}
+            foreach item [concat [lsort -dictionary -index 1 $items] \
+				 [lsort -dictionary -index 1 $items_with_nodes]] {
+                lappend neworder [lindex $item 0]
+            }
+            reorder_node $tw $tnode $neworder
+
+            foreach child [$tw nodes $tnode] {
+                browser::browser_action $action $bw $child
+            }
+	}
+
+	default {
+	}
+    }
+}
+
 # TODO
 proc disco::browser::textballoon {bw action X Y node} {
     variable disco
