@@ -5,7 +5,7 @@ LD_LIBRARY_PATH=/usr/pkg/lib:/usr/lib:/usr/local/lib \
 export PATH LD_LIBRARY_PATH; exec tclsh "$0" "$@"
 
 
-package require jabberlib
+package require jabberlib -exact 0.9.0
 package require http 2
 package require mime
 package require tls
@@ -24,6 +24,7 @@ proc jlib::sendit {stayP to args} {
                             -from        ""       \
                             -password    ""       \
                             -host        ""       \
+                            -port        ""       \
                             -activity    ""       \
                             -type        headline \
                             -subject     ""       \
@@ -95,16 +96,33 @@ proc jlib::sendit {stayP to args} {
         }
     }
 
-    if {![string compare $lib(sckstats) off]} {
+    if {[llength $lib(connections)] <= 0} {
+	set connid [jlib::new -user $aprops(local) \
+			      -server $aprops(domain) \
+			      -resource $aprops(resource)]
         if {$options(-tls)} {
-            set sd [tls::socket $aprops(domain) 5223]
+	    set transport tls
+	    if {[string compare $options(-port) ""]} {
+		set port $options(-port)
+	    } else {
+		set port 5223
+	    }
         } else {
-            set sd [socket $aprops(domain) 5222]
+	    set transport tcp
+	    if {[string compare $options(-port) ""]} {
+		set port $options(-port)
+	    } else {
+		set port 5222
+	    }
         }
-        jlib::connect $sd $aprops(domain)
-        jlib::wait_for_stream
+        jlib::connect $connid \
+		      -transport $transport \
+		      -host $aprops(domain) \
+		      -port $port
+        jlib::wait_for_stream $connid
     
-        jlib::send_auth $aprops(local) $options(-password) $aprops(resource) \
+        jlib::send_auth $connid \
+	    $aprops(local) $options(-password) $aprops(resource) \
             [namespace current]::sendit_aux
         vwait [namespace current]::sendit_result
     
@@ -156,7 +174,7 @@ proc jlib::roster_get_aux {what} {
     set sendit_result $what
 }
 
-proc client:message {from id type subject body err thread priority x} {
+proc client:message {connid from id type is_subject subject body err thread priority x} {
     ::LOG "client:message $from $body"
 
     if {![regexp {(.*@.*)/.*} $from x jid]} {
@@ -192,7 +210,7 @@ proc client:message {from id type subject body err thread priority x} {
     }
 }
 
-proc client:presence {from type x args} {
+proc client:presence {connid from type x args} {
     ::LOG "client:presence $args"
 
     if {![regexp {(.*@.*)/.*} $from x jid]} {
@@ -217,7 +235,7 @@ proc client:presence {from type x args} {
     rssbot::presence $jid $type
 }
 
-proc client:iqreply {from userid id type child} {
+proc client:iqreply {connid from userid id type child} {
     jlib::wrapper:splitxml $child tag vars isempty chdata children
     set xmlns [jlib::wrapper:getattr $vars xmlns]
     ::LOG "client:iqreply $from $userid $id $type $xmlns"
@@ -287,11 +305,11 @@ proc client:roster_push {args} {}
 
 proc client:roster_item {args} {}
 
-proc client:reconnect {} {
+proc client:reconnect {connid} {
     rssbot::reconnect
 }
 
-proc client:disconnect {} {
+proc client:disconnect {connid} {
     rssbot::reconnect
 }
 
