@@ -1,9 +1,9 @@
-# $Id: filetransfer.tcl,v 1.27 2004/07/10 20:05:04 aleksey Exp $
+# $Id: filetransfer.tcl,v 1.28 2005/03/03 01:55:50 aleksey Exp $
 
 namespace eval ft {
     
     custom::defgroup FileTransfer [::msgcat::mc "File Transfer options."] \
-	-group Tkabber
+	-group Tkabber -tag "File Transfer"
 
     switch -- $::tcl_platform(platform) {
 	windows {
