@@ -1,43 +1,34 @@
-# $Id: gpgme.tcl,v 1.23 2004/07/15 21:56:10 aleksey Exp $
+# $Id: gpgme.tcl,v 1.29 2005/07/31 21:09:36 aleksey Exp $
+
+if {[catch { package require gpgme }]} {
+    debugmsg ssj "unable to load the GPGME package, so no crypto!"
+    set have_gpgme 0
+    return
+} else {
+    set have_gpgme 1
+}
 
 namespace eval ssj {
     variable options
 
-    if {![info exists options(one-passphrase)]} {
-        set options(one-passphrase) 0
-    }
+    custom::defgroup GPGME [::msgcat::mc "GPGME options (signing and encryption)."] \
+	-group Tkabber
 
-    if {[llength [array names options encrypt,*]] > 0} {
-        set needP 1
-    } else {
-        set needP 0
-    }
-    if {![info exists options(sign-traffic)]} {
-        if {[info exists options(encrypt-traffic)]} {
-            set options(sign-traffic) 1
-        } else {
-            set options(sign-traffic) $needP
-        }
-    }
+    custom::defvar options(one-passphrase) 1 \
+	[::msgcat::mc "Use the same passphrase for signing and decrypting messages."] \
+	-group GPGME -type boolean
 
-    if {![info exists options(encrypt-traffic)]} {
-        set options(encrypt-traffic) $options(sign-traffic)
-    }
+    custom::defvar options(sign-traffic) 0 \
+	[::msgcat::mc "GPG-sign outgoing messages and presence updates."] \
+	-group GPGME -type boolean
 
-    if {[catch { package require gpgme }]} {
-        if {($options(sign-traffic)) || ($options(encrypt-traffic))} {
-            set needP 1
-        }
+    custom::defvar options(encrypt-traffic) 0 \
+	[::msgcat::mc "GPG-encrypt outgoing messages where possible."] \
+	-group GPGME -type boolean
 
-        if {$needP} {
-            debugmsg ssj \
-                     "unable to load the PGPME package, so no crypto!
-
-See contrib/pgpme for details"
-        }
-
-        return
-    }
+    custom::defvar options(key) "" \
+	[::msgcat::mc "Use specified key ID for signing and decrypting messages."] \
+	-group GPGME -type string
 }
 
 
@@ -55,9 +46,7 @@ namespace eval ssj {
     variable warnings
     variable gpg_error_id 0
 
-    if {![info exists ctx]} {
-        set ctx ""
-    }
+    array set ctx {}
 
     array set j2k {}
 
@@ -80,46 +69,51 @@ namespace eval ssj {
 }
 
 
-proc ssj::once_only {{armorP 0}} {
-    global env loginconf
-
+proc ssj::once_only {connid {armorP 0}} {
+    global env
+    variable options
     variable ctx
-    variable selectkey
 
-    if {![cequal $ctx ""]} {
-        $ctx -operation set   \
-             -property  armor \
-             -value     $armorP
+    if {[info exists ctx($connid)] && ![cequal $ctx($connid) ""]} {
+        $ctx($connid) -operation set   \
+		      -property  armor \
+		      -value     $armorP
 
         return
     }
 
-    set ctx [gpgme::context]
-    $ctx -operation set   \
-         -property  armor \
-         -value     $armorP
+    set ctx($connid) [gpgme::context]
+    $ctx($connid) -operation set   \
+		  -property  armor \
+		  -value     $armorP
 
 
     if {![info exists env(GPG_AGENT_INFO)]} {
-        $ctx -operation set                 \
-             -property  passphrase-callback \
-             -value     [namespace current]::passphrase
+        $ctx($connid) -operation set                 \
+		      -property  passphrase-callback \
+		      -value     [list [namespace current]::passphrase $connid]
     }
 
-    set pattern [jlib::connection_bare_jid [jlib::route ""]]
+    set pattern [jlib::connection_bare_jid $connid]
 
     set firstP 1
-    foreach p [list $pattern ""] {
-        set command [list $ctx -operation start-key -secretonly true]
+    if {$options(key) != ""} {
+	set patterns [list $options(key)]
+    } else {
+	set patterns {}
+    }
+    lappend patterns $pattern ""
+    foreach p $patterns {
+        set command [list $ctx($connid) -operation start-key -secretonly true]
         if {![cequal $p ""]} {
             lappend command -patterns [list $p]
         }
         eval $command
 
         for {set keys {}} \
-            {![cequal [set key [$ctx -operation next-key]] ""]} \
+            {![cequal [set key [$ctx($connid) -operation next-key]] ""]} \
             {lappend keys $key} {}
-        $ctx -operation done-key
+        $ctx($connid) -operation done-key
 
         if {[llength $keys] > 0} {
             break
@@ -137,7 +131,7 @@ proc ssj::once_only {{armorP 0}} {
 
         1 {
             if {$firstP} {
-                e4meP $keys
+                e4meP $connid $keys
                 return
             }
         }
@@ -146,13 +140,13 @@ proc ssj::once_only {{armorP 0}} {
         }
     }
 
-    set dw .selectkey
+    set dw .selectkey$connid
     catch { destroy $dw }
 
     set titles {}
     set balloons {}
     foreach key $keys {
-        foreach {k v} [$ctx -operation info-key -key $key] {
+        foreach {k v} [$ctx($connid) -operation info-key -key $key] {
             if {![cequal $k subkeys]} {
                 continue
             }
@@ -175,20 +169,19 @@ proc ssj::once_only {{armorP 0}} {
 	}
     }
 
-    CbDialog $dw [::msgcat::mc "Select Key for Signing Traffic"] \
-        [list [::msgcat::mc "Select"] "[namespace current]::once_only_aux $dw" \
+    CbDialog $dw [format [::msgcat::mc "Select Key for Signing %s Traffic"] $pattern] \
+        [list [::msgcat::mc "Select"] "[namespace current]::once_only_aux $dw $connid" \
 	      [::msgcat::mc "Cancel"] "destroy $dw"] \
-	[namespace current]::selectkey $titles $balloons \
+	[namespace current]::selectkey$connid $titles $balloons \
 	-modal local
 }
 
-proc ssj::once_only_aux {dw} {
-    variable ctx
-    variable selectkey
+proc ssj::once_only_aux {dw connid} {
+    variable selectkey$connid
 
     set keys {}
-    foreach key [array names selectkey] {
-        if {$selectkey($key)} {
+    foreach key [array names selectkey$connid] {
+        if {[set selectkey${connid}($key)]} {
             lappend keys $key
         }
     }
@@ -196,13 +189,12 @@ proc ssj::once_only_aux {dw} {
     destroy $dw
 
     if {[llength $keys] > 0} {
-        e4meP $keys
+        e4meP $connid $keys
     }
 }
 
 
-proc ssj::passphrase {data} {
-    variable ctx
+proc ssj::passphrase {connid data} {
     variable passphrase
     variable options
 
@@ -222,12 +214,12 @@ proc ssj::passphrase {data} {
     }
 
     if {([cequal $text ENTER]) \
-            &&([info exists passphrase($keyid)]) \
+            && ([info exists passphrase($keyid)]) \
             && (![cequal $passphrase($keyid) ""])} {
         return $passphrase($keyid)
     }
 
-    set pw .passphrase
+    set pw .passphrase$connid
     if {[winfo exists $pw]} {
         destroy $pw
     }
@@ -250,7 +242,8 @@ proc ssj::passphrase {data} {
     set pf [$pw getframe]
     grid columnconfigure $pf 1 -weight 1
 
-    foreach {k v} [list keyid [::msgcat::mc "Key ID"] userid [::msgcat::mc "User ID"]] {
+    foreach {k v} [list keyid  [::msgcat::mc "Key ID"] \
+			userid [::msgcat::mc "User ID"]] {
         label $pf.l$k -text ${v}:
         entry $pf.$k
         $pf.$k insert 0 [set $k]
@@ -266,9 +259,10 @@ proc ssj::passphrase {data} {
     }
 
     label $pf.lpassword -text [::msgcat::mc "Passphrase:"]
-    entry $pf.password  -textvariable [namespace current]::passphrase($keyid) \
-                        -show *
-    set passphrase($keyid) ""
+    entry $pf.password  \
+	  -textvariable [namespace current]::passphrase($connid,$keyid) \
+          -show *
+    set passphrase($connid,$keyid) ""
 
     grid $pf.lkeyid    -row 0 -column 0 -sticky e
     grid $pf.keyid     -row 0 -column 1 -sticky ew
@@ -282,11 +276,14 @@ proc ssj::passphrase {data} {
 
     if {[set abort [$pw draw $pf.password]]} {
         $params(token) -operation cancel
+	# TODO: unset options(sign-traffic) etc. ?
     }
 
     destroy $pw
 
     if {!$abort} {
+	set passphrase($keyid) $passphrase($connid,$keyid)
+	unset passphrase($connid,$keyid)
         return $passphrase($keyid)
     }
 }
@@ -307,26 +304,26 @@ proc ssj::armor:decode {text} {
     return "-----BEGIN PGP MESSAGE-----\n\n$text\n-----END PGP MESSAGE-----"
 }
 
-
-proc ssj::signed:input {from signature data what} {
+proc ssj::signed:input {connid from signature data what} {
     variable ctx
     variable j2k
     variable s2e
     variable warnings
     variable gpg_error_id
 
-    once_only
+    once_only $connid
 
-    if {[catch { $ctx -operation verify \
-                      -input     [binary format a* [encoding convertto utf-8 $data]]  \
-                      -signature [armor:decode $signature] } result]} {
-        debugmsg ssj "verify processing error: $result ($from)"
+    if {[catch { $ctx($connid) -operation verify \
+			       -input     [binary format a* [encoding convertto utf-8 $data]]  \
+			       -signature [armor:decode $signature] } result]} {
+        debugmsg ssj "verify processing error ($connid): $result ($from)"
 
-        if {![info exists warnings(verify-traffic)]} {
+        if {![info exists warnings(verify-traffic,$connid)]} {
 
-            set warnings(verify-traffic) 1
-            after idle [list NonmodalMessageDlg .verify_error -aspect 50000 -icon error \
-                -message [format [::msgcat::mc "Error in signature verification software: %s."] $result]]
+            set warnings(verify-traffic,$connid) 1
+            after idle [list NonmodalMessageDlg .verify_error$connid -aspect 50000 -icon error \
+                -message [format [::msgcat::mc "Error in signature verification software: %s."] \
+				 $result]]
         }
 
         set params(reason) $result
@@ -334,7 +331,7 @@ proc ssj::signed:input {from signature data what} {
         return [array get params]
     }
 
-    debugmsg ssj "VERIFY: $from ($data); $result"
+    debugmsg ssj "VERIFY: $connid $from ($data); $result"
 
     array set params $result
     set result $params(status)
@@ -345,7 +342,7 @@ proc ssj::signed:input {from signature data what} {
         array set sparams $signature
 
         if {[info exists sparams(key)]} {
-            set sparams(key) [$ctx -operation info-key -key $sparams(key)]
+            set sparams(key) [$ctx($connid) -operation info-key -key $sparams(key)]
             foreach {k v} $sparams(key) {
                 if {![cequal $k subkeys]} {
                     continue
@@ -376,7 +373,9 @@ proc ssj::signed:input {from signature data what} {
             set warnings(verify,$from) 1
             incr gpg_error_id
             after idle [list NonmodalMessageDlg .verify_error$gpg_error_id -aspect 50000 -icon error \
-                -message [format [::msgcat::mc "%s purportedly signed by %s can't be verified.\n\n%s."] $what $from $result]]
+                -message [format \
+			      [::msgcat::mc "%s purportedly signed by %s can't be verified.\n\n%s."] \
+			      $what $from $result]]
         }
     }
 
@@ -384,7 +383,7 @@ proc ssj::signed:input {from signature data what} {
 }
 
 
-proc ssj::signed:output {data args} {
+proc ssj::signed:output {connid data args} {
     variable ctx
     variable options
     variable warnings
@@ -394,14 +393,14 @@ proc ssj::signed:output {data args} {
         return
     }
 
-    once_only 1
+    once_only $connid 1
 
-    if {[catch { $ctx -operation sign  \
-                      -input     [binary format a* [encoding convertto utf-8 $data]] \
-                      -mode      detach } result]} {
+    if {[catch { $ctx($connid) -operation sign  \
+			       -input     [binary format a* [encoding convertto utf-8 $data]] \
+			       -mode      detach } result]} {
         set options(sign-traffic) 0
 
-        debugmsg ssj "signature processing error: $result ($data)"
+        debugmsg ssj "signature processing error ($connid): $result ($data)"
 
         if {[llength $args] == 0} {
             set buttons ok
@@ -425,7 +424,7 @@ proc ssj::signed:output {data args} {
     set result [armor:encode $result]
 
     debugmsg ssj "SIGN: $data; $result"
-    whichkeys sign
+    whichkeys $connid sign
 
     return $result
 }
@@ -469,6 +468,11 @@ proc ssj::signed:info {pinfo} {
 		    append info "created: [clock format $v]\n"
 		}
 
+		fingerprint {
+		    append info [format "keyid: 0x%s\n" [string range $v end-7 end]]
+		    append info [format "%s: %s\n" $k $v]
+		}
+
 		default {
 		    if {![cequal $v ""]} {
 			append info [format "%s: %s\n" $k $v]
@@ -502,33 +506,95 @@ proc ssj::signed:Label {lb pinfo} {
 	lappend args -helptext $info -helptype balloon
     }
 
-    return [eval [list Label $lb] $args -cursor arrow \
-		-padx 0 -pady 0 -borderwidth 0 -highlightthickness 0]
+    eval [list Label $lb] $args -cursor arrow \
+	 -padx 0 -pady 0 -borderwidth 0 -highlightthickness 0
+
+    if {[info exists params(reason)] && [cequal $params(reason) nokey]} {
+	bind $lb <3> [list [namespace current]::signed:popup $pinfo]
+    }
+    return $lb
 }
 
+proc ssj::signed:popup {pinfo} {
+    set m .signed_label_popupmenu
+    if {[winfo exists $m]} {
+	destroy $m
+    }
+    menu $m -tearoff 0
+    $m add command -label [::msgcat::mc "Fetch GPG key"] \
+	-command [list [namespace current]::fetchkeys $pinfo]
+    tk_popup $m [winfo pointerx .] [winfo pointery .]
+}
 
-proc ssj::encrypted:input {from data} {
+proc ssj::signed:user_menu {m connid jid} {
+    global presence
+    global curuser
+
+    if {[cequal $jid "\$curuser"]} {
+	set jid $curuser
+    }
+    if {[info exists presence(signed,$connid,$jid)]} {
+	array set params $presence(signed,$connid,$jid)
+	if {[info exists params(status)] && [cequal $params(status) nokey]} {
+	    $m add command -label [::msgcat::mc "Fetch GPG key"] \
+		-command [list [namespace current]::fetchkeys \
+			       $presence(signed,$connid,$jid)]
+	}
+    }
+}
+
+hook::add roster_create_user_menu_edit_hook \
+    [namespace current]::ssj::signed:user_menu 60
+
+proc ssj::fetchkeys {pinfo} {
+    variable gpg_error_id
+
+    array set params $pinfo
+
+    set keyids {}        
+    foreach signature $params(signatures) {
+	catch { unset sparams }
+	array set sparams $signature
+
+	if {[info exists sparams(fingerprint)]} {
+	    lappend keyids [string range $sparams(fingerprint) end-7 end]
+	}
+    }
+    set res [catch {set output [eval [list exec gpg --recv-keys] $keyids]} errMsg]
+    incr gpg_error_id
+    if {$res} {
+        NonmodalMessageDlg .keyfetch_ok$gpg_error_id -aspect 50000 -icon error \
+            -message "Key fetch error\n\n$errMsg"
+    } else {
+        NonmodalMessageDlg .keyfetch_error$gpg_error_id -aspect 50000 -icon info \
+            -message "Key fetch result\n\n$output"
+    }
+}
+
+proc ssj::encrypted:input {connid from data} {
     variable ctx
     variable warnings
     variable gpg_error_id
 
-    once_only
+    once_only $connid
 
-    if {[catch { $ctx -operation decrypt \
-                      -input     [armor:decode $data] } result]} {
-        debugmsg ssj "decryption processing error: $result ($from)"
+    if {[catch { $ctx($connid) -operation decrypt \
+			       -input     [armor:decode $data] } result]} {
+        debugmsg ssj "decryption processing error ($connid): $result ($from)"
 
         if {![info exists warnings(decrypt,$from)]} {
             set warnings(decrypt,$from) 1
             incr gpg_error_id
             after idle [list NonmodalMessageDlg .decrypt_error$gpg_error_id -aspect 50000 -icon error \
-                -message [format [::msgcat::mc "Data purported sent by %s can't be deciphered.\n\n%s."] $from $result]]
+                -message [format \
+			      [::msgcat::mc "Data purported sent by %s can't be deciphered.\n\n%s."] \
+			      $from $result]]
         }
 
         error $result
     }
 
-    debugmsg ssj "DECRYPT: $from; $result"
+    debugmsg ssj "DECRYPT: $connid; $from; $result"
 
     array set params $result
     binary scan $params(plaintext) a* temp_utf8
@@ -536,7 +602,7 @@ proc ssj::encrypted:input {from data} {
 }
 
 
-proc ssj::encrypted:output {data to} {
+proc ssj::encrypted:output {connid data to} {
     variable ctx
     variable e4me
     variable j2k
@@ -551,7 +617,7 @@ proc ssj::encrypted:output {data to} {
     #    set jid $to
     #}
     set jid [node_and_server_from_jid $to]
-    if {![encryptP $jid]} {
+    if {![encryptP $connid $jid]} {
         return
     }
 
@@ -562,28 +628,29 @@ proc ssj::encrypted:output {data to} {
     } else {
         set name $jid
     }
-    [set recipient [gpgme::recipient]] \
-            -operation add   \
-            -name      $name \
-            -validity  full
-    foreach signer $e4me {
+
+    set recipient [gpgme::recipient]
+    $recipient -operation add   \
+	       -name      $name \
+               -validity  full
+    foreach signer $e4me($connid) {
         $recipient -operation add \
                    -name      $signer \
                    -validity  full
     }
 
-    once_only 1
+    once_only $connid 1
 
-    set code [catch { $ctx -operation   encrypt \
-                           -input       [binary format a* [encoding convertto utf-8 $data]]   \
-                           -recipients $recipient } result]
+    set code [catch { $ctx($connid) -operation   encrypt \
+				    -input       [binary format a* [encoding convertto utf-8 $data]] \
+				    -recipients $recipient } result]
 
     rename $recipient {}
 
     if {$code} {
-        debugmsg ssj "encryption processing error: $result ($data)"
+        debugmsg ssj "encryption processing error ($connid): $result ($data)"
 
-        set options(encrypt,$jid) 0
+        set options(encrypt,$connid,$jid) 0
         incr gpg_error_id
         if {[MessageDlg .encrypt_error$gpg_error_id -aspect 50000 -icon error -type user \
                 -buttons {ok cancel} -default 0 -cancel 1 \
@@ -595,16 +662,17 @@ proc ssj::encrypted:output {data to} {
     }
     set result [armor:encode $result]
 
-    debugmsg ssj "ENCRYPT: $data; $result"
+    debugmsg ssj "ENCRYPT: $connid; $data; $result"
 
     return $result
 }
 
-proc ssj::whichkeys {what} {
+proc ssj::whichkeys {connid what} {
     variable ctx
     variable warnings
 
-    if {[cequal [set s [$ctx -operation get -property  last-op-info]] ""]} {
+    set s [$ctx($connid) -operation get -property last-op-info]
+    if {[cequal $s ""]} {
         return
     }
 
@@ -627,49 +695,64 @@ proc ssj::whichkeys {what} {
 }
 
 
-proc ssj::prefs {jid} {
+proc ssj::prefs {connid jid} {
     variable ctx
     variable options
     variable optionsX
 
-    set w [win_id security_preferences $jid]
+    set w [win_id security_preferences [list $connid $jid]]
 
     if {[winfo exists $w]} {
         focus -force $w
         return
     }
 
-    Dialog $w -title [format [::msgcat::mc "Change security preferences for %s"] $jid] -separator 1 \
-        -anchor e -default 0 -cancel 1
+    Dialog $w \
+	   -title [format [::msgcat::mc "Change security preferences for %s"] $jid] \
+	   -separator 1 -anchor e -default 0 -cancel 1
+
+    $w add -text [::msgcat::mc "OK"] \
+	   -command [list [namespace current]::prefs_ok $w $connid $jid]
+    $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
 
     set f [$w getframe]
 
-    if {![info exists options(encrypt,$jid)]} {
-        set options(encrypt,$jid) [encryptP $jid]
+    if {![info exists options(encrypt,$connid,$jid)]} {
+        set options(encrypt,$connid,$jid) [encryptP $connid $jid]
     }
 
-    set optionsX(encrypt,$jid) $options(encrypt,$jid)
+    set optionsX(encrypt,$connid,$jid) $options(encrypt,$connid,$jid)
     checkbutton $f.encrypt \
         -text     [::msgcat::mc "Encrypt traffic"] \
-        -variable [namespace current]::optionsX(encrypt,$jid)
+        -variable [namespace current]::optionsX(encrypt,$connid,$jid)
 
-    pack $f.encrypt -side left -expand yes -fill x
-
-    $w add -text [::msgcat::mc "OK"] -command [list [namespace current]::prefs_ok $w $jid]
-    $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
+    pack $f.encrypt -side left
+    pack [frame $f.f -width 9c -height 2c]
 
     $w draw $f.name
 }
 
-proc ssj::prefs_ok {w jid} {
+proc ssj::prefs_ok {w connid jid} {
     variable options
     variable optionsX
 
-    set options(encrypt,$jid) $optionsX(encrypt,$jid)
+    set options(encrypt,$connid,$jid) $optionsX(encrypt,$connid,$jid)
 
     destroy $w
 }
 
+proc ssj::prefs_user_menu {m connid jid} {
+    global curuser
+
+    if {[cequal $jid "\$curuser"]} {
+	set jid $curuser
+    }
+    $m add command -label [::msgcat::mc "Edit security..."] \
+	-command [list [namespace current]::prefs $connid $jid]
+}
+
+hook::add roster_create_user_menu_edit_hook \
+    [namespace current]::ssj::prefs_user_menu
 
 proc ssj::signP {} {
     variable options
@@ -677,17 +760,16 @@ proc ssj::signP {} {
     return $options(sign-traffic)
 }
 
-proc ssj::encryptP {{jid ""}} {
+proc ssj::encryptP {connid jid} {
     variable ctx
     variable j2k
     variable options
 
     if {[cequal $jid ""]} {
-        return $options(encrypt-traffic)
+	return $options(encrypt-traffic)
     }
 
-    lassign [roster::get_category_and_subtype \
-                 [jlib::route $jid] $jid] \
+    lassign [roster::get_category_and_subtype $connid $jid] \
             category subtype
     switch -- $category {
 	conference
@@ -701,19 +783,21 @@ proc ssj::encryptP {{jid ""}} {
 	}
     }
 
-    if {[info exists options(encrypt,$jid)]} {
-        return $options(encrypt,$jid)
+    if {[info exists options(encrypt,$connid,$jid)]} {
+        return $options(encrypt,$connid,$jid)
+    } elseif {[info exists options(encrypt,$jid)]} {
+	return $options(encrypt,$jid)
     }
 
     if {!$options(encrypt-traffic)} {
         return 0
     }
 
-    if {[info exists options(encrypt-tried,$jid)]} {
-        return $options(encrypt-tried,$jid)
+    if {[info exists options(encrypt-tried,$connid,$jid)]} {
+        return $options(encrypt-tried,$connid,$jid)
     }
 
-    once_only
+    once_only $connid
 
     if {[info exists j2k($jid)]} {
         set name $j2k($jid)
@@ -728,40 +812,40 @@ proc ssj::encryptP {{jid ""}} {
             -name      $name \
             -validity  full
 
-    if {[catch { $ctx -operation  encrypt        \
-                      -input      "Hello world." \
-                      -recipients $recipient }]} {
-        set options(encrypt-tried,$jid) 0
+    if {[catch { $ctx($connid) -operation  encrypt        \
+			       -input      "Hello world." \
+			       -recipients $recipient }]} {
+        set options(encrypt-tried,$connid,$jid) 0
     } else {
-        set options(encrypt-tried,$jid) 1
+        set options(encrypt-tried,$connid,$jid) 1
     }
 
     rename $recipient {}
 
-    return $options(encrypt-tried,$jid)
+    return $options(encrypt-tried,$connid,$jid)
 }
 
 
-proc ssj::e4meP {keys} {
+proc ssj::e4meP {connid keys} {
     variable ctx
     variable e4me
     variable signers
 
-    $ctx -operation set     \
-         -property  signers \
-         -value     [set signers $keys]
+    $ctx($connid) -operation set     \
+		  -property  signers \
+		  -value     [set signers($connid) $keys]
 
-    set e4me {}
-    foreach signer $signers {
+    set e4me($connid) {}
+    foreach signer $signers($connid) {
         [set recipient [gpgme::recipient]] \
                 -operation add     \
                 -name      $signer \
                 -validity  full
 
-        if {![catch { $ctx -operation  encrypt       \
-                          -input      "Hello world." \
-                          -recipients $recipient } result]} {
-            lappend e4me $signer
+        if {![catch { $ctx($connid) -operation  encrypt        \
+				    -input      "Hello world." \
+				    -recipients $recipient } result]} {
+            lappend e4me($connid) $signer
         }
 
         rename $recipient {}
@@ -771,21 +855,25 @@ proc ssj::e4meP {keys} {
 proc ssj::sign:toggleP {} {
     variable options
 
-    set options(sign-traffic) [expr ($options(sign-traffic)+1)%2]
+    set options(sign-traffic) [expr {!$options(sign-traffic)}]
 }
 
-proc ssj::encrypt:toggleP {{jid ""}} {
+proc ssj::encrypt:toggleP {{connid ""} {jid ""}} {
     variable options
 
     if {[cequal $jid ""]} {
-        set options(encrypt-traffic) [expr ($options(encrypt-traffic)+1)%2]
+	set options(encrypt-traffic) [expr {!$options(encrypt-traffic)}]
         return
     }
 
-    if {![info exists options(encrypt,$jid)]} {
-        set options(encrypt,$jid) [encryptP $jid]
+    if {![cequal $connid ""]} {
+	if {![info exists options(encrypt,$connid,$jid)]} {
+	    set options(encrypt,$connid,$jid) [encryptP $connid $jid]
+	}
+	set options(encrypt,$connid,$jid) [expr {!$options(encrypt,$connid,$jid)}]
+    } else {
+	return -code error "ssj::encrypt:toggleP: connid is empty and jid is not"
     }
-    set options(encrypt,$jid) [expr ($options(encrypt,$jid)+1)%2]
 }
 
 
@@ -796,21 +884,25 @@ proc ssj::signed:trace {script} {
     if {![info exists trace(sign-traffic)]} {
         set trace(sign-traffic) {}
 
-        ::trace variable options(sign-traffic) w \
+        ::trace variable [namespace current]::options(sign-traffic) w \
                 [namespace current]::trace
     }
 
     lappend trace(sign-traffic) $script
 }
 
-proc ssj::encrypted:trace {script {jid ""}} {
+proc ssj::encrypted:trace {script {connid ""} {jid ""}} {
     variable options
     variable trace
 
     if {[cequal $jid ""]} {
-        set k encrypt-traffic
+	set k encrypt-traffic
     } else {
-        set k encrypt,$jid 
+	if {![cequal $connid ""]} {
+	    set k encrypt,$connid,$jid
+	} else {
+	    return -code error "ssj::encrypted:trace: connid is empty and jid is not"
+	}
     }
     if {![info exists trace($k)]} {
         set trace($k) {}
