@@ -1,4 +1,4 @@
-# $Id: iface.tcl,v 1.21 2004/07/15 21:56:10 aleksey Exp $
+# $Id: iface.tcl,v 1.36 2005/08/06 22:13:32 aleksey Exp $
 
 namespace eval ifacetk {
     variable options
@@ -12,21 +12,31 @@ namespace eval ifacetk {
 	     "Use Tabbed Interface (you need to restart)."] \
 	-group IFace -type boolean
 
+    custom::defvar options(tabs_at_bottom) 0 \
+	[::msgcat::mc "Show tabs below the main window."] \
+	-group IFace -type boolean \
+	-command [namespace current]::configure_tabs
+
     custom::defvar options(show_toolbar) 1 \
 	[::msgcat::mc "Show Toolbar."] \
 	-group IFace -type boolean \
-	-command ifacetk::switch_toolbar
+	-command [namespace current]::switch_toolbar
 
     custom::defvar options(show_presencebar) 1 \
 	[::msgcat::mc "Show presence bar."] \
 	-group IFace -type boolean \
-	-command ifacetk::switch_presencebar
+	-command [namespace current]::switch_presencebar
+
+    custom::defvar options(show_statusbar) 1 \
+	[::msgcat::mc "Show status bar."] \
+	-group IFace -type boolean \
+	-command [namespace current]::switch_statusbar
 
     if {$::tcl_platform(platform) != "unix"} {
 	custom::defvar options(font) $::default_font \
 	    [::msgcat::mc "Font to use in roster, chat windows etc."] \
 	    -group IFace -type font \
-	    -command ifacetk::switch_font
+	    -command [namespace current]::switch_font
     }
 
     custom::defvar options(raise_new_tab) 1 \
@@ -52,15 +62,20 @@ namespace eval ifacetk {
 		     close    [::msgcat::mc "Close Tkabber"] \
 		     nothing  [::msgcat::mc "Do nothing"] \
 		     minimize [::msgcat::mc "Minimize"] \
+		     iconize  [::msgcat::mc "Iconize"] \
 		    ]
 
+    custom::defvar mainwindowstate normal \
+	[::msgcat::mc "Stored main window state (normal or zoomed)"] \
+	-type string -group Hidden
+
     variable toolbaricon
 
-    set toolbaricon(groupchat) [Bitmap::get [pixmap tkabber glade-groupchat.gif]]
-    set toolbaricon(adduser)   [Bitmap::get [pixmap tkabber glade-new-user.gif]]
-    set toolbaricon(browser)   [Bitmap::get [pixmap tkabber jb.gif]]
-    set toolbaricon(online)    [Bitmap::get [pixmap tkabber glade-online.gif]]
-    set toolbaricon(offline)   [Bitmap::get [pixmap tkabber glade-offline.gif]]
+    set toolbaricon(groupchat) [Bitmap::get [pixmap tkabber toolbar-join-conference.gif]]
+    set toolbaricon(adduser)   [Bitmap::get [pixmap tkabber toolbar-add-user.gif]]
+    set toolbaricon(disco)     [Bitmap::get [pixmap tkabber toolbar-disco.gif]]
+    set toolbaricon(online)    [Bitmap::get [pixmap tkabber toolbar-show-online.gif]]
+    set toolbaricon(offline)   [Bitmap::get [pixmap tkabber toolbar-show-offline.gif]]
 
     # TODO
     set ::searchicon(case)       [Bitmap::get [pixmap tkabber search_case.gif]]
@@ -83,26 +98,63 @@ namespace eval ifacetk {
     variable personal_msg
 }
 
+proc ifacetk::configure_tabs {args} {
+    variable options
+    global usetabbar
+
+    if {$usetabbar} {
+	if {$options(tabs_at_bottom)} {
+	    set side bottom
+	} else {
+	    set side top
+	}
+	.nb configure -side $side
+    }
+}
+
+proc ifacetk::save_mainwindowstate {w} {
+    variable mainwindowstate
+
+    if {![cequal $w .]} return
+
+    set state [wm state .]
+    if {$state != $mainwindowstate} {
+	set mainwindowstate $state
+	set custom::saved([namespace current]::mainwindowstate) $mainwindowstate
+	custom::store
+    }
+}
+
+hook::add map_window_hook [namespace current]::ifacetk::save_mainwindowstate
+
 option add *errorForeground red widgetDefault
 
 wm protocol . WM_DELETE_WINDOW {
-    ifacetk::closebuttonproc
+    hook::run protocol_wm_delete_window_hook
 }
 
 proc ifacetk::closebuttonproc {} { 
     variable options
 
     switch -- $options(closebuttonaction) {
-        close {
-	    terminate
-        }
         nothing {}
         minimize {
 	    wm iconify .
         }
+	iconize {
+	    wm iconify .
+	    wm state . withdrawn
+	}
+        close -
+	default {
+	    quit
+        }
     }
 }
 
+hook::add protocol_wm_delete_window_hook \
+    [namespace current]::ifacetk::closebuttonproc
+
 set xselection ""
 
 proc EncodeTextSelection {txt offset len} {
@@ -177,11 +229,11 @@ proc ifacetk::load_descmenu {} {
     global descmenu
 
     set descmenu \
-	[list "&Tkabber" {} {} $ifacetk::options(show_tearoffs) \
+	[list "&Tkabber" {} tkabber $ifacetk::options(show_tearoffs) \
 	     [list \
 		  [list cascad [::msgcat::mc "Presence"] {} presence $ifacetk::options(show_tearoffs) \
 		       [list \
-			    [list command [::msgcat::mc "Online"] {} {} {} \
+			    [list command [::msgcat::mc "Available"] {} {} {} \
 				 -command {set userstatus available}] \
 			    [list command [::msgcat::mc "Free to chat"] {} {} {} \
 				 -command {set userstatus chat}] \
@@ -197,12 +249,73 @@ proc ifacetk::load_descmenu {} {
 			    [list command [::msgcat::mc "Change priority..."] {} {} {} \
 				 -command {change_priority_dialog}]]] \
 		  [list command [::msgcat::mc "Log in..."] {} {} {Ctrl l} \
-		       -command {ifacetk::login_dialog}] \
+		       -command [list [namespace current]::login_dialog]] \
 		  [list command [::msgcat::mc "Log out"] {} {} {Ctrl j} \
-		       -command {logout_dialog}] \
+		       -command [list [namespace current]::logout_dialog]] \
 		  [list command [::msgcat::mc "Log out with reason..."] {} {} {} \
 		       -command {show_logout_dialog}] \
 		  {separator} \
+		  [list command [::msgcat::mc "Change password..."] {} {} {} \
+		       -command {change_password_dialog}] \
+		  [list command [::msgcat::mc "Edit my info..."] {} {} {} \
+		       -command {
+			    set connid [jlib::route ""]
+			    userinfo::open [jlib::connection_bare_jid $connid] \
+				-editable 1 -connection $connid}] \
+		  [list cascad [::msgcat::mc "Privacy rules"] {} {} $ifacetk::options(show_tearoffs) \
+		       [list \
+			    [list command [::msgcat::mc "Edit invisible list..."] {} {} {} \
+				 -command {privacy::edit_special_list invisible}] \
+			    [list command [::msgcat::mc "Edit ignore list..."] {} {} {} \
+				 -command {privacy::edit_special_list ignore}] \
+			    [list command [::msgcat::mc "Manually edit rules..."] {} {} {} \
+				 -command {privacy::request_lists}] \
+			    [list checkbutton [::msgcat::mc "Activate lists at startup"] {} {} {} \
+				 -variable privacy::options(activate_at_startup)] \
+			    {separator} \
+			    [list command [::msgcat::mc "Message filters..."] {} {} {} \
+				 -command {filters::open}]]] \
+		  {separator} \
+		  [list cascad [::msgcat::mc "View"] {} {} $ifacetk::options(show_tearoffs) \
+		   [list \
+		     [list checkbutton [::msgcat::mc "Toolbar"] \
+			  {} {} {} \
+			  -variable [namespace current]::options(show_toolbar)] \
+		     [list checkbutton [::msgcat::mc "Presence bar"] \
+			  {} {} {} \
+			  -variable [namespace current]::options(show_presencebar)] \
+		     [list checkbutton [::msgcat::mc "Status bar"] \
+			  {} {} {} \
+			  -variable [namespace current]::options(show_statusbar)] \
+		  ]] \
+		  [list cascad [::msgcat::mc "Roster"] {} roster $ifacetk::options(show_tearoffs) \
+		   [list \
+		    [list checkbutton [::msgcat::mc "Show online users only"] \
+			 {} {} {} \
+			 -variable [namespace current]::roster::show_only_online \
+			 -command [namespace current]::roster::redraw_after_idle] \
+		    [list checkbutton [::msgcat::mc "Use aliases"] {} {} {} \
+			 -variable [namespace current]::roster::use_aliases] \
+		    [list cascad [::msgcat::mc "Export roster..."] \
+			 export_roster export_roster 0 {}] \
+		    [list cascad [::msgcat::mc "Import roster..."] \
+			 import_roster import_roster 0 {}] \
+		    [list command [::msgcat::mc "Add group by regexp on JIDs..."] {} {} {} \
+			 -command [namespace current]::roster::add_group_by_jid_regexp_dialog] \
+		  ]] \
+		  [list cascad [::msgcat::mc "Chats"] {} chats $ifacetk::options(show_tearoffs) \
+		   [list \
+		    [list checkbutton [::msgcat::mc "Generate event messages"] {} {} {} \
+			 -variable muc::options(gen_events)] \
+		    [list checkbutton [::msgcat::mc "Smart autoscroll"] {} {} {} \
+			 -variable chat::options(smart_scroll)] \
+		    [list checkbutton [::msgcat::mc "Stop autoscroll"] {} {} {} \
+			 -variable chat::options(stop_scroll)] \
+		    [list checkbutton [::msgcat::mc "Emphasize"] {} {} {} \
+			 -variable chat::options(emphasize)] \
+		    {separator} \
+		  ]] \
+		  {separator} \
 		  [list command [::msgcat::mc "Customize"] {} {} {} \
 		       -command {custom::open_window Tkabber}] \
 		  {separator} \
@@ -217,64 +330,29 @@ proc ifacetk::load_descmenu {} {
 		  [list command [::msgcat::mc "Quit"] {} {} {} \
 		       -command {quit}] \
 		 ] \
-	     "&[::msgcat::mc Services]" {} {} $ifacetk::options(show_tearoffs) \
+	     "&[::msgcat::mc Services]" {} services $ifacetk::options(show_tearoffs) \
 	     [list \
+	      [list command [::msgcat::mc "Add user to roster..."] {} {} {} \
+		       -command {message::send_subscribe_dialog ""}] \
+	      [list command [::msgcat::mc "Add conference to roster..."] {} {} {} \
+		       -command {plugins::conferences::add_conference_dialog}] \
+		  {separator} \
 		  [list command [::msgcat::mc "Send message..."] {} {} {} \
 		       -command {message::send_dialog}] \
-		  $::roster_main_menu \
-		  [list command [::msgcat::mc "Browser"] {} {} {} \
-		       -command {browser::open}] \
-		  [list command [::msgcat::mc "Discovery"] {} {} {} \
-		       -command {disco::browser::open_win ""}] \
 		  [list command [::msgcat::mc "Join group..."] {} {} {}\
 		       -command {join_group_dialog}] \
-		  [list cascad [::msgcat::mc "Sound"] {} {} $ifacetk::options(show_tearoffs) \
-		       [list \
-			    [list checkbutton [::msgcat::mc "Mute"] {} {} {} \
-				 -variable sound::options(mute)]]] \
-		  [list cascad [::msgcat::mc "Chats"] {} chats $ifacetk::options(show_tearoffs) \
-		       [list \
-			    [list checkbutton [::msgcat::mc "Generate event messages"] {} {} {} \
-				 -variable muc::options(gen_events)] \
-			    [list checkbutton [::msgcat::mc "Smart autoscroll"] {} {} {} \
-				 -variable chat::options(smart_scroll)] \
-			    [list checkbutton [::msgcat::mc "Stop autoscroll"] {} {} {} \
-				 -variable chat::options(stop_scroll)] \
-			    [list checkbutton [::msgcat::mc "Emphasize"] {} {} {} \
-				 -variable chat::options(emphasize)]]] \
-		  [list cascad [::msgcat::mc "Privacy rules"] {} {} $ifacetk::options(show_tearoffs) \
-		       [list \
-			    [list command [::msgcat::mc "Manually edit rules..."] {} {} {} \
-				 -command {privacy::request_lists}] \
-			    [list command [::msgcat::mc "Edit invisible list..."] {} {} {} \
-				 -command {privacy::edit_special_list invisible}] \
-			    [list command [::msgcat::mc "Edit ignore list..."] {} {} {} \
-				 -command {privacy::edit_special_list ignore}] \
-			    [list checkbutton [::msgcat::mc "Activate lists at startup"] {} {} {} \
-				 -variable privacy::options(activate_at_startup)] \
-			    {separator} \
-			    [list command [::msgcat::mc "Message filters..."] {} {} {} \
-				 -command {filters::open}]]] \
-		  [list command [::msgcat::mc "Change password..."] {} {} {} \
-		       -command {change_password_dialog}] \
-		  [list command [::msgcat::mc "Message archive"] {} {} {} \
-		       -command {message_archive::show_archive}] \
 		  [list command [::msgcat::mc "Show user info..."] {} {} {} \
 		       -command {userinfo::show_info_dialog}] \
-		  [list command [::msgcat::mc "Edit my info..."] {} {} {} \
-		       -command {
-			    set connid [jlib::route ""]
-			    userinfo::open [jlib::connection_bare_jid $connid] \
-				-editable 1 -connection $connid
-			}] \
-		  [list cascad [::msgcat::mc "Avatar"] {} {} $ifacetk::options(show_tearoffs) \
-		       [list \
-			    [list checkbutton [::msgcat::mc "Announce"] {} {} {} \
-				 -variable avatar::options(announce)] \
-			    [list checkbutton [::msgcat::mc "Allow downloading"] {} {} {} \
-				 -variable avatar::options(share)] \
-			    [list command [::msgcat::mc "Send to server"] {} {} {} \
-				 -command {avatar::store_on_server}]]] \
+		  {separator} \
+		  [list command [::msgcat::mc "Jabber Browser"] {} {} {} \
+		       -command {browser::open}] \
+		  [list command [::msgcat::mc "Service Discovery"] {} {} {} \
+		       -command {disco::browser::open_win ""}] \
+		  {separator} \
+		  [list command [::msgcat::mc "Message archive"] {} {} {} \
+		       -command {message_archive::show_archive}] \
+          [list cascad [::msgcat::mc "Plugins"] {} plugins $ifacetk::options(show_tearoffs) \
+	        ] \
 		  {separator} \
 		  [list cascad [::msgcat::mc "Admin tools"] {} admin $ifacetk::options(show_tearoffs) \
 		       [list \
@@ -289,7 +367,9 @@ proc ifacetk::load_descmenu {} {
 						     -to "$loginconf(server)/announce/motd/update"}}] \
 			    [list command [::msgcat::mc "Delete message of the day"] {} {} {} \
 				 -command {eval {message::send_msg \
-						     "$loginconf(server)/announce/motd/delete" -type normal}}]]] \
+						     "$loginconf(server)/announce/motd/delete" -type normal}}] \
+		        {separator} \
+               ]] \
 		 ] \
 	     "&[::msgcat::mc Help]" {} {} $ifacetk::options(show_tearoffs) \
 	     [list \
@@ -340,7 +420,8 @@ proc ifacetk::quick_help_window {} {
     $::tk_modify-Z\t\t\t[::msgcat::mc {Undo}]
     $::tk_modify-Shift-Z\t\t[::msgcat::mc {Redo}]
     Alt-PgUp/Down\t\t[::msgcat::mc {Scroll chat window up/down}]
-    [::msgcat::mc {Right mouse	}]\t[::msgcat::mc {Correct word}]
+    $::tk_modify-S\t\t\t[::msgcat::mc {Search in chat window}]
+    [::msgcat::mc {Right mouse button}]\t[::msgcat::mc {Correct word}]
 "
 }
 
@@ -350,7 +431,7 @@ proc ifacetk::about_window {} {
     help_window [::msgcat::mc "About"] "
 Tkabber $version
 
-Copyright \u00a9 2002-2004 [::msgcat::mc {Alexey Shchepin}]
+Copyright \u00a9 2002-2005 [::msgcat::mc {Alexey Shchepin}]
 \n[::msgcat::mc Authors:]
     [::msgcat::mc {Alexey Shchepin}]
     [::msgcat::mc {Marshall T. Rose}]
@@ -361,7 +442,7 @@ http://tkabber.jabberstudio.org/"
 }
 
 proc ifacetk::help_window {title message} {
-    set mainlogo [Bitmap::get [pixmap tkabber mainlogo.gif]]
+    set mainlogo [Bitmap::get [pixmap tkabber tkabber-logo.gif]]
     Dialog .m -anchor e -separator yes -title $title -image $mainlogo \
 	    -side bottom -modal local -default 0 -cancel 0
     .m add -text [::msgcat::mc "OK"]
@@ -544,16 +625,13 @@ proc ifacetk::create_main_window {} {
 
     pack $mf -expand yes -fill both
 
-    bind $mf <Destroy> {
-	jlib::disconnect
-	quit
-    }
+    bind $mf <Destroy> quit
 
     set bbox [ButtonBox [$mf addtoolbar].bbox -spacing 0 -padx 1 -pady 1]
 
     add_toolbar_button $bbox $toolbaricon(adduser) {message::send_subscribe_dialog ""} \
 	[::msgcat::mc "Add new user..."]
-    add_toolbar_button $bbox $toolbaricon(browser) {disco::browser::open_win ""} \
+    add_toolbar_button $bbox $toolbaricon(disco) {disco::browser::open_win ""} \
 	[::msgcat::mc "Service Discovery"]
     add_toolbar_button $bbox $toolbaricon(groupchat) join_group_dialog \
 	[::msgcat::mc "Join group..."]
@@ -624,17 +702,11 @@ proc ifacetk::create_main_window {} {
     }
 
     switch_presencebar
-
-    init_toolbar_popup_menu
-
-    catch {
-	bind [$mf gettoolbar 0] <3> [list tk_popup .toolbar_popup %X %Y]
-	bind $mf.status <3> [list tk_popup .toolbar_popup %X %Y]
-    }
+    switch_statusbar
 
     set rosterwidth [option get . mainRosterWidth [winfo class .]]
     if {$rosterwidth == ""} {
-	set rosterwidth [winfo pixels . 4c]
+	set rosterwidth [winfo pixels . 3c]
     }
 
     if {$usetabbar} {
@@ -648,11 +720,18 @@ proc ifacetk::create_main_window {} {
 	    -grouppopup [namespace current]::roster::group_popup_menu
 	pack .roster -expand yes -fill both -side left -in $rw
 
-	NoteBook .nb -width 400
+	if {$options(tabs_at_bottom)} {
+	    set side bottom
+	} else {
+	    set side top
+	}
+
+	NoteBook .nb -width 400 -side $side
 	pack .nb -side right -in $nw -fill both -expand yes
 	[winfo parent $rw] configure -width $rosterwidth
 
-	bind . <Control-Key-r>      [list [namespace current]::collapse_roster]
+	event add <<CollapseRoster>> <Control-Key-r>
+	bind . <<CollapseRoster>>      [list [namespace current]::collapse_roster]
 	bind . <Control-Prior>      [list [namespace current]::tab_move .nb -1]
 	bind . <Control-Next>       [list [namespace current]::tab_move .nb 1]
 	bind . <Control-Meta-Prior> [list [namespace current]::current_tab_move .nb -1]
@@ -722,6 +801,8 @@ proc ifacetk::create_main_window {} {
 	}
     }
 
+    bind . <Map> [list hook::run map_window_hook .]
+
     wm geometry . $geometry
 
     if {$usetabbar} {
@@ -730,6 +811,7 @@ proc ifacetk::create_main_window {} {
     }
 
     define_alert_colors
+    update idletasks
 }
 
 proc ifacetk::draginitcmd {target x y top} {
@@ -779,21 +861,6 @@ proc ifacetk::dropcmd {target source X Y op type data} {
     }
 }
 
-proc ifacetk::init_toolbar_popup_menu {} {
-    set m .toolbar_popup
-
-    if {[winfo exists $m]} {
-	destroy $m
-    }
-
-    menu $m -tearoff 0
-
-    $m add checkbutton -label [::msgcat::mc "Show toolbar"] \
-	-variable [namespace current]::options(show_toolbar)
-    $m add checkbutton -label [::msgcat::mc "Show presence bar"] \
-	-variable [namespace current]::options(show_presencebar)
-}
-
 proc ifacetk::destroy_win {path} {
     global usetabbar
 
@@ -938,7 +1005,7 @@ proc ifacetk::add_number_of_messages_to_title {chatid from type body extras} {
     } elseif {$type == "groupchat"} {
 	set jid [chat::get_jid $chatid]
 	set myjid [chat::our_jid $chatid]
-	set mynick [chat::get_nick $myjid $type]
+	set mynick [chat::get_nick [chat::get_connid $chatid] $myjid $type]
 	if {![cequal $myjid $from]} {
 	    incr number_msg($chatid)
 	}
@@ -1016,6 +1083,7 @@ proc ifacetk::add_win {path args} {
 	}
     } else {
 	toplevel $path -class $class -relief flat -bd 2m
+	wm group $path .
 	wm title $path $title
 	wm iconname $path $title
 	set geometry [option get $path ${type}geometry $class]
@@ -1141,9 +1209,6 @@ proc ifacetk::define_alert_colors {} {
 		[option get .nb alertColor1 NoteBook] \
 		[option get .nb alertColor2 NoteBook] \
 		[option get .nb alertColor3 NoteBook]]
-	custom::defvar alert_colors $alert_colors \
-	    [::msgcat::mc "Colors for tab alert levels."] \
-	    -group IFace -type string
     }
 }
 
@@ -1241,24 +1306,46 @@ proc ifacetk::switch_toolbar {args} {
 }
 
 proc ifacetk::switch_presencebar {args} {
+    global usetabbar
     variable options
     variable mf
 
     if {[winfo exists .presence]} {
 	if {$options(show_presencebar)} {
-	    pack .presence -side bottom -anchor w \
-		-in [.mainframe getframe] -fill x
+	    if {$usetabbar} {
+		set before [.mainframe getframe].pw
+	    } else {
+		set before .roster
+	    }
+	    catch {
+		pack .presence -side bottom -anchor w \
+		    -in [.mainframe getframe] -fill x -before $before
+	    }
 	} else {
 	    pack forget .presence
 	}
     }
 }
 
-proc ifacetk::deiconify {} {
-    global usetabbar tcl_platform
+proc ifacetk::switch_statusbar {args} {
+    variable options
+    variable mf
 
-    if {[cequal $tcl_platform(platform) "windows"] && $usetabbar} {
-	if {[cequal [option get . mainwindowstate Tkabber] "zoomed"]} {
+    catch {
+	if {$options(show_statusbar)} {
+	    pack $mf.botf -side bottom -fill x -before [$mf getframe]
+	} else {
+	    pack forget $mf.botf
+	}
+    }
+}
+
+proc ifacetk::deiconify {} {
+    global tcl_platform
+    variable mainwindowstate
+
+    if {[cequal $tcl_platform(platform) windows]} {
+	if {[cequal $mainwindowstate zoomed]} {
 	    wm state . zoomed
 	}
     }
@@ -1267,3 +1354,12 @@ proc ifacetk::deiconify {} {
 
 hook::add finload_hook [namespace current]::ifacetk::deiconify 99
 
+proc client:errormsg {message} {
+    if {[winfo exists .client_error]} {
+	destroy .client_error
+    }
+    
+    MessageDlg .client_error -aspect 50000 -icon error \
+	-message $message -type user -buttons ok -default 0 -cancel 0
+}
+
