@@ -1,4 +1,4 @@
-# $Id: iroster.tcl,v 1.20 2004/07/10 20:05:04 aleksey Exp $
+# $Id: iroster.tcl,v 1.35 2005/08/19 20:13:19 aleksey Exp $
 
 Tree .faketree
 foreach {k v} [list background       White \
@@ -134,59 +134,67 @@ proc roster::set_group_lists {connid} {
     variable undef_group_name
     variable chats_group_name
 
-    if {[catch { set bare_jid [jlib::connection_bare_jid $connid] }]} {
-	return
+    if {$connid == {}} {
+	set connidlist [jlib::connections]
+    } else {
+	set connidlist [list $connid]
     }
-    if {[catch { set groups [roster::get_groups $connid -raw 1] }]} {
-	return
-    }
-    if {$options(nested)} {
-	set tmp {}
+
+    foreach connid $connidlist {
+	if {[catch { set bare_jid [jlib::connection_bare_jid $connid] }]} {
+	    continue
+	}
+	if {[catch { set groups [roster::get_groups $connid -raw 1] }]} {
+	    continue
+	}
+	if {$options(nested)} {
+	    set tmp {}
+	    foreach group $groups {
+		set tmp1 [::textutil::splitx $group $options(nested_delimiter)]
+		for {set i 0} {$i < [llength $tmp1]} {incr i} {
+		    lappend tmp [lrange $tmp1 0 $i]
+		}
+	    }
+	    set groups [lrmdups $tmp]
+	}
+	lappend groups $undef_group_name $chats_group_name
+	set tmpc {}
+	set tmps {}
 	foreach group $groups {
-	    set tmp1 [::textutil::splitx $group $options(nested_delimiter)]
-	    for {set i 0} {$i < [llength $tmp1]} {incr i} {
-		lappend tmp [lrange $tmp1 0 $i]
+	    if {$options(nested)} {
+		set grname [join $group $options(nested_delimiter)]
+		set gid [list $connid $group]
+	    } else {
+		set grname [list $group]
+		set gid [list $connid $grname]
+	    }
+	    if {[info exists roster(collapsed,$gid)]} {
+		set collapsed($bare_jid,$grname) $roster(collapsed,$gid)
+		lappend tmpc $grname $roster(collapsed,$gid)
+	    }
+	    if {[info exists roster(show_offline,$gid)]} {
+		set show_offline($bare_jid,$grname) $roster(show_offline,$gid)
+		lappend tmps $grname $roster(show_offline,$gid)
 	    }
 	}
-	set groups [lrmdups $tmp]
+	if {![lempty $tmpc]} {
+	    while {[set idx [lsearch -exact $collapsed_group_list $bare_jid]] >= 0} {
+		set collapsed_group_list \
+		    [lreplace $collapsed_group_list $idx [expr {$idx + 1}]]
+	    }
+	    lappend collapsed_group_list $bare_jid $tmpc
+	}
+	if {![lempty $tmps]} {
+	    while {[set idx [lsearch -exact $show_offline_group_list $bare_jid]] >= 0} {
+		set show_offline_group_list \
+		    [lreplace $show_offline_group_list $idx [expr {$idx + 1}]]
+	    }
+	    lappend show_offline_group_list $bare_jid $tmps
+	}
+	set custom::saved([namespace current]::collapsed_group_list) $collapsed_group_list
+	set custom::saved([namespace current]::show_offline_group_list) $show_offline_group_list
+	custom::store
     }
-    lappend groups $undef_group_name $chats_group_name
-    set tmpc {}
-    set tmps {}
-    foreach group $groups {
-	if {$options(nested)} {
-	    set grname [join $group $options(nested_delimiter)]
-	    set gid [list $connid $group]
-	} else {
-	    set grname [list $group]
-	    set gid [list $connid $grname]
-	}
-	if {[info exists roster(collapsed,$gid)]} {
-	    set collapsed($bare_jid,$grname) $roster(collapsed,$gid)
-	    lappend tmpc $grname $roster(collapsed,$gid)
-	}
-	if {[info exists roster(show_offline,$gid)]} {
-	    set show_offline($bare_jid,$grname) $roster(show_offline,$gid)
-	    lappend tmps $grname $roster(show_offline,$gid)
-	}
-    }
-    if {![lempty $tmpc]} {
-	while {[set idx [lsearch -exact $collapsed_group_list $bare_jid]] >= 0} {
-	    set collapsed_group_list \
-		[lreplace $collapsed_group_list $idx [expr {$idx + 1}]]
-	}
-	lappend collapsed_group_list $bare_jid $tmpc
-    }
-    if {![lempty $tmps]} {
-	while {[set idx [lsearch -exact $show_offline_group_list $bare_jid]] >= 0} {
-	    set show_offline_group_list \
-		[lreplace $show_offline_group_list $idx [expr {$idx + 1}]]
-	}
-	lappend show_offline_group_list $bare_jid $tmps
-    }
-    set custom::saved([namespace current]::collapsed_group_list) $collapsed_group_list
-    set custom::saved([namespace current]::show_offline_group_list) $show_offline_group_list
-    custom::store
 }
 
 hook::add predisconnected_hook [namespace current]::roster::set_group_lists
@@ -216,14 +224,6 @@ proc roster::redraw {} {
 
     clear .roster 0
     
-    if {$use_aliases} {
-	foreach jid [array names aliases] {
-	    foreach alias $aliases($jid) {
-		set ignore_jid($alias) ""
-	    }
-	}
-    }
-
     set connections [jlib::connections]
     switch -- [llength $connections] {
 	0 {
@@ -241,6 +241,49 @@ proc roster::redraw {} {
     }
 
     foreach connid $connections {
+        # Suppress display of aliases, but not if "main" JID is unavailable.
+        # If main JID is unavailable, perform reordering of aliases.
+        # Suppression means that aliases wont get a line in the roster for
+	# themselves, but will be places as children under "main" JID element
+	if {$use_aliases} {
+	    foreach jid [array names aliases] {
+		set status [get_user_status $connid $jid]
+		if {$status == "unavailable"} {
+		    # Need to look an alternative for main JID,
+		    # which is offline.
+		    set jidstatus [get_user_aliases_status_and_jid $connid $jid]
+                    switch -- [lindex $jidstatus 1] {
+                        unavailable {
+                            # Main JID and all aliases are unavailable.
+			    # Will skip them all
+                        }
+                        default {
+                            # Make "most available" JID the main JID,
+			    # put main JID into list of aliases
+                            set new_jid [lindex $jidstatus 0]
+                            set idx [lsearch -exact $aliases($jid) $new_jid]
+                            set new_aliases [lreplace $aliases($jid) $idx $idx]
+                            # If this already have some aliases defined,
+			    # merge them in
+                            if {[info exists aliases($new_jid)]} {
+                                set new_aliases \
+				    [lsort -unique -dictionary -index 0 \
+					 [lappend new_aliases \
+					      $aliases($new_jid)]]
+                            }
+                            set aliases($new_jid) $new_aliases
+                            # Remove aliases of old "main" JID
+                            set aliases($jid) {}
+                        }
+                    }
+                }
+                # Main JID is online, suppress alternatives
+                foreach alias $aliases($jid) {
+                    set ignore_jid($alias) ""
+                }
+            }
+        }
+        
 	if {$draw_connection} {
 	    if {![info exists roster(collapsed,[list connid $connid])]} {
 		set roster(collapsed,[list connid $connid]) 0
@@ -272,12 +315,12 @@ proc roster::redraw {} {
 		    } else {
 			set sgroup [list $group]
 		    }
-		    lappend groups [join $sgroup "\u0000"]
+		    lappend groups [list [join $sgroup "\u0000"] $sgroup]
 		    lappend jidsingroup($sgroup) $jid
 		    set deep [expr {[llength $sgroup] - 1}]
 		    for {set i 0} {$i < $deep} {incr i} {
 			set sgr [lrange $sgroup 0 $i]
-			lappend groups [join $sgr "\u0000"]
+			lappend groups [list [join $sgr "\u0000"] $sgr]
 			lappend jidsundergroup($sgr) $jid
 			lappend groupsundergroup($sgr) $sgroup
 			if {![info exists jidsingroup($sgr)]} {
@@ -300,10 +343,10 @@ proc roster::redraw {} {
 		}
 	    }
 	}
-	set groups [lsort -unique -dictionary $groups]
+	set groups [lsort -unique -dictionary -index 0 $groups]
 	set ugroup [list $undef_group_name]
 	if {[info exists jidsingroup($ugroup)]} {
-	    lappend groups [join $ugroup "\u0000"]
+	    lappend groups [list [join $ugroup "\u0000"] $ugroup]
 	}
 	if {$options(chats_group)} {
 	    set cgroup [list $chats_group_name]
@@ -314,19 +357,20 @@ proc roster::redraw {} {
 		    lappend jidsingroup($cgroup) $jid
 		    if {[cequal [roster::itemconfig $connid $jid -isuser] ""]} {
 			roster::itemconfig $connid $jid -isuser 1
-			roster::itemconfig $connid $jid -name [chat::get_nick $jid chat]
+			roster::itemconfig $connid $jid \
+			    -name [chat::get_nick $connid $jid chat]
 			roster::itemconfig $connid $jid -subsc none
 		    }
 		}
 	    }
 	    if {[info exists jidsingroup($cgroup)]} {
-		set groups [linsert $groups 0 [join $cgroup "\u0000"]]
+		set groups [linsert $groups 0 [list [join $cgroup "\u0000"] $cgroup]]
 	    }
 	    set groupsundergroup($cgroup) {}
 	    set jidsundergroup($cgroup) {}
 	}
 	foreach group $groups {
-	    set group [split $group "\u0000"]
+	    set group [lindex $group 1]
 	    set gid [list $connid $group]
 	    if {![info exists roster(show_offline,$gid)]} {
 		if {$options(nested)} {
@@ -342,7 +386,7 @@ proc roster::redraw {} {
 	    }
 	}
 	foreach group $groups {
-	    set group [split $group "\u0000"]
+	    set group [lindex $group 1]
 	    set jidsingroup($group) [lrmdups $jidsingroup($group)]
 	    set groupsundergroup($group) [lrmdups $groupsundergroup($group)]
 	    set gid [list $connid $group]
@@ -360,10 +404,11 @@ proc roster::redraw {} {
 	    }
 	    set indent [expr {[llength $group] - 1}]
 	    set collapse 0
-	    set show_offlin 0
+	    set show_offline_users 0
+	    set show_offline_group 0
 	    foreach undergroup $groupsundergroup($group) {
 		if {$roster(show_offline,[list $connid $undergroup])} {
-		    set show_offlin 1
+		    set show_offline_group 1
 		    break
 		}
 	    }
@@ -374,7 +419,8 @@ proc roster::redraw {} {
 		    break
 		}
 		if {$roster(show_offline,$sgr)} {
-		    set show_offlin 1
+		    set show_offline_users 1
+		    set show_offline_group 1
 		}
 	    }
 	    incr indent $gindent
@@ -399,7 +445,7 @@ proc roster::redraw {} {
 		    incr not_users
 		}
 	    }
-	    if {!$show_only_online || $show_offlin || \
+	    if {!$show_only_online || $show_offline_group || \
 		    $roster(show_offline,$gid) || \
 		    $online + $not_users + $sub_jids > 0} {
 		if {$users} {
@@ -426,7 +472,7 @@ proc roster::redraw {} {
 			}
 		    }
 		    set cjid [list $connid $jid]
-		    if {!$show_only_online || $show_offlin || $roster(show_offline,$gid) || \
+		    if {!$show_only_online || $show_offline_users || $roster(show_offline,$gid) || \
 			    ![info exists useronline($jid)] || $useronline($jid)} {
 			lassign [::roster::get_category_and_subtype $connid $jid] category type
 			set jids [get_jids_of_user $connid $jid]
@@ -521,18 +567,28 @@ proc roster::get_jids_of_user {connid user} {
 }
 
 proc roster::get_user_aliases_status {connid user} {
+    set jidstatus [get_user_aliases_status_and_jid $connid $user]
+    return [lindex $jidstatus 1]
+}
+
+proc roster::get_user_aliases_status_and_jid {connid user} {
     upvar #0 roster::aliases aliases
     variable use_aliases
 
     if {$use_aliases && [info exists aliases($user)]} {
 	set status [get_user_status $connid $user]
+        set jid $user
 
 	foreach alias $aliases($user) {
-	    set status [max_status $status [get_user_status $connid $alias]]
+	    set new_status [max_status $status [get_user_status $connid $alias]]
+            if {$status != $new_status} {
+                set status $new_status
+                set jid $alias
 	}
-	return $status
+	}
+	return [list $jid $status]
     } else {
-	return [get_user_status $connid $user]
+	return [list $user [get_user_status $connid $user]]
     }
 }
 
@@ -545,8 +601,7 @@ proc roster::get_jid_foreground {connid jid} {
 	    return [get_user_foreground $connid $jid]
 	}
 	conference {
-	    set status [get_jid_status $connid $jid]
-	    if {$status != "unavailable"} {
+	    if {[get_jid_status $connid $jid] != "unavailable"} {
 		return available
 	    } else {
 		return unavailable
@@ -574,8 +629,7 @@ proc roster::get_jid_icon {connid jid} {
 	    return [get_user_icon $connid $jid]
 	}
 	conference {
-	    set show [get_jid_status $connid $jid]
-	    if {$show != {unavailable}} {
+	    if {[get_jid_status $connid $jid] != "unavailable"} {
 		return $conferenceicon(available)
 	    }
 	    return $conferenceicon(unavailable)
@@ -974,10 +1028,14 @@ proc roster::jid_doubleclick {id} {
 
     switch -- $category {
 	conference {
-	    global gr_nick
-	    join_group $jid \
-		-nick [get_group_nick $jid $gr_nick] \
-		-connection $connid
+	    if {[roster::itemconfig $connid $jid -subsc] == "bookmark"} {
+		plugins::conferences::join_group $connid $jid
+	    } else {
+		global gr_nick
+		join_group $jid \
+		    -nick [get_group_nick $jid $gr_nick] \
+		    -connection $connid
+	    }
 	}
 	user -
 	default {
@@ -1023,20 +1081,22 @@ proc roster::jids_popup_info {id jids} {
 proc roster::user_popup_info {connid user i} {
     variable options
     variable user_popup_info
+    global statusdesc
 
     lassign [::roster::get_category_and_subtype $connid $user] category subtype
     set bare_user [node_and_server_from_jid $user]
-    lassign [::roster::get_category_and_subtype $connid $bare_user] category1 subtype1
+    lassign [::roster::get_category_and_subtype $connid $bare_user] \
+	category1 subtype1
 
     set name $user
     switch -- $category {
 	conference {
-	    set status [get_jid_status $connid $user]
+	    set status $statusdesc([get_jid_status $connid $user])
 	    set desc ""
 	}
 	user -
 	default {
-	    set status [get_user_status $connid $user]
+	    set status $statusdesc([get_user_status $connid $user])
 	    set desc   [get_user_status_desc $connid $user]
 	    if {[cequal $category1 conference] && $i > 0} {
 		if {$options(show_conference_user_info)} {
@@ -1048,7 +1108,7 @@ proc roster::user_popup_info {connid user i} {
 	}
     }
 
-    if {(![cequal $status [string tolower $desc]]) && (![cequal $desc ""])} {
+    if {(![string equal -nocase  $status $desc]) && (![cequal $desc ""])} {
 	append status " ($desc)"
     }
 
@@ -1062,7 +1122,8 @@ proc roster::user_popup_info {connid user i} {
 
     set user_popup_info "$name$subsc: $status"
 
-    if {!([cequal $category1 conference] && $i > 0) || $options(show_conference_user_info)} {
+    if {!([cequal $category1 conference] && $i > 0) || \
+	    $options(show_conference_user_info)} {
 	hook::run roster_user_popup_info_hook \
 	    [namespace which -variable user_popup_info] $connid $user
     }
@@ -1077,7 +1138,7 @@ proc roster::switch_only_online {args} {
 }
 
 proc roster::is_online {connid jid} {
-    if {[is_user $connid $jid]} {
+    if {[::roster::is_user $connid $jid]} {
 	switch -- [get_user_aliases_status $connid $jid] {
 	    unavailable {return 0}
 	    default {return 1}
@@ -1125,8 +1186,8 @@ proc roster::dropcmd {target source X Y op type data} {
     set yc [$c canvasy $y]
 
     set tags [$c gettags [lindex [$c find closest $xc $yc] 0]]
-    
-    if {$options(free_drop)} {
+
+    if {$options(free_drop) && ![cequal $tags ""]} {
 	lassign [tag_to_jid [crange [lindex $tags 1] 5 end]] connid gr
 	if {$connid == "connid"} {
 	    set connid $gr
@@ -1150,7 +1211,16 @@ proc roster::dropcmd {target source X Y op type data} {
 
     debugmsg roster "GG: $gr; $tags"
 
-    lassign $data jid category type name version
+    lassign $data jid category type name version fromgid
+
+    if {[info exists fromgid]} {
+	lassign $fromgid fromconnid fromgr 
+	if {$options(nested)} {
+	    set fromgr [join $fromgr $options(nested_delimiter)]
+	} else {
+	    set fromgr [lindex $fromgr 0]
+	}
+    }	
 
     if {![lcontain [::roster::get_jids $connid] $jid]} {
 	if {$gr != {}} {
@@ -1168,15 +1238,27 @@ proc roster::dropcmd {target source X Y op type data} {
 	}
     } else {
 	set groups [::roster::itemconfig $connid $jid -group]
+
+	if {[info exists fromgid] && ($fromconnid == $connid)} {
+	    set idx [lsearch -exact $groups $fromgr]
+	    if {$idx >= 0} {
+		set groups [lreplace $groups $idx $idx]
+	    }
+	}
 	if {$gr != ""} {
 	    lappend groups $gr
 	    set groups [lrmdups $groups]
 	    debugmsg roster $groups
 	}
+
 	::roster::itemconfig $connid $jid -category $category -subtype $type \
 	    -name $name -group $groups
     }
-    ::roster::send_item $connid $jid
+    if {[::roster::itemconfig $connid $jid -subsc] == "bookmark"} {
+	plugins::conferences::update_bookmark $connid $jid -groups $groups
+    } else {
+	::roster::send_item $connid $jid
+    }
 }
 
 proc roster::draginitcmd {target x y top} {
@@ -1187,6 +1269,8 @@ proc roster::draginitcmd {target x y top} {
 
     set tags [$c gettags current]
     if {[lcontain $tags jid]} {
+	set grouptag [crange [lindex $tags 1] 5 end]
+	set gid [tag_to_jid $grouptag]
 	set tag [crange [lindex $tags 0] 3 end]
 	set cjid [tag_to_jid $tag]
 	lassign $cjid connid jid
@@ -1194,10 +1278,11 @@ proc roster::draginitcmd {target x y top} {
 	set data [list $jid \
 		      [::roster::itemconfig $connid $jid -category] \
 		      [::roster::itemconfig $connid $jid -subtype] \
-		      [::roster::itemconfig $connid $jid -name] {}]
+		      [::roster::itemconfig $connid $jid -name] {} \
+		      $gid]
 
 	debugmsg roster $data
-	return [list JID {copy} $data]
+	return [list JID {move} $data]
     } else {
 	return {}
     }
@@ -1213,11 +1298,16 @@ proc roster::popup_menu {id} {
 
     lassign [::roster::get_category_and_subtype $connid $jid] category subtype
 
-    switch -- $category {
-	conference {set menu [conference_popup_menu $connid $jid]}
-	user       {set menu [create_user_menu $connid $jid]}
-	service    {set menu [service_popup_menu $connid $jid]}
-	default    {set menu [jid_popup_menu $connid $jid]}
+    if {[roster::itemconfig $connid $jid -subsc] == "bookmark"} {
+	set menu [plugins::conferences::popup_menu $connid $jid]
+	common_menu $menu $connid $jid
+    } else {
+	switch -- $category {
+	    conference {set menu [conference_popup_menu $connid $jid]}
+	    user       {set menu [create_user_menu $connid $jid]}
+	    service    {set menu [service_popup_menu $connid $jid]}
+	    default    {set menu [jid_popup_menu $connid $jid]}
+	}
     }
 
     tk_popup $menu [winfo pointerx .] [winfo pointery .]
@@ -1342,20 +1432,16 @@ proc roster::create_user_menu {connid user} {
 
 
     $m add separator
-    add_menu_item $m [::msgcat::mc "Show info..."] \
+    add_menu_item $m [::msgcat::mc "Show info"] \
 	"userinfo::open \$curuser -connection [list $connid]" $jids
-    $m add command -label [::msgcat::mc "Show history..."] \
+    $m add command -label [::msgcat::mc "Show history"] \
 	-command {logger::show_log $curuser}
     $m add separator
-    $m add command -label [::msgcat::mc "Edit item..."] \
-	-command [list itemedit::show_dialog $connid $user]
-    # TODO: use $connid
-    $m add command -label [::msgcat::mc "Edit security..."] \
-	-command {ssj::prefs $curuser} \
-	-state [lindex {normal disabled} \
-		    [cequal [info commands ::ssj::prefs] ""]]
+
+    hook::run roster_create_user_menu_edit_hook $m $connid "\$curuser"
+
     $m add separator
-    $m add command -label [::msgcat::mc "Remove..."] \
+    $m add command -label [::msgcat::mc "Remove item..."] \
 	-command [list [namespace current]::remove_item_dialog $connid $user]
 
     return $m
@@ -1375,8 +1461,8 @@ namespace eval roster {
     set rostericon(group,opened)     [Bitmap::get [pixmap roster group-opened.gif]]
     set rostericon(group,closed)     [Bitmap::get [pixmap roster group-closed.gif]]
 
-    set conferenceicon(available)    [Bitmap::get [pixmap roster group_on.gif]]
-    set conferenceicon(unavailable)  [Bitmap::get [pixmap roster group_off.gif]]
+    set conferenceicon(available)    [Bitmap::get [pixmap roster conference-available.gif]]
+    set conferenceicon(unavailable)  [Bitmap::get [pixmap roster conference-unavailable.gif]]
 
     set serviceicon(aim,available)   [Bitmap::get [pixmap services aim_online.xpm]]
     set serviceicon(aim,chat)        [Bitmap::get [pixmap services aim_chat.xpm]]
@@ -1423,29 +1509,26 @@ namespace eval roster {
     set serviceicon(sms) [Bitmap::get [pixmap services sms.xpm]]
     set serviceicon(jud) [Bitmap::get [pixmap services jud.gif]]
 
-    set serviceicon(rss,available)   [Bitmap::get [pixmap services rss.xpm]]
-    set serviceicon(rss,away)        [Bitmap::get [pixmap services rss2.xpm]]
-    set serviceicon(rss,xa)          [Bitmap::get [pixmap services rss2.xpm]]
-    set serviceicon(rss,dnd)         [Bitmap::get [pixmap services rss3.xpm]]
-    set serviceicon(rss,unavailable) [Bitmap::get [pixmap services rss2.xpm]]
+    set serviceicon(rss,available)   [Bitmap::get [pixmap services rss_online.xpm]]
+    set serviceicon(rss,chat)        [Bitmap::get [pixmap services rss_chat.xpm]]
+    set serviceicon(rss,away)        [Bitmap::get [pixmap services rss_away.xpm]]
+    set serviceicon(rss,xa)          [Bitmap::get [pixmap services rss_xa.xpm]]
+    set serviceicon(rss,dnd)         [Bitmap::get [pixmap services rss_dnd.xpm]]
+    set serviceicon(rss,unavailable) [Bitmap::get [pixmap services rss_offline.xpm]]
 }
 
 proc roster::common_menu {m connid jid} {
     $m add separator
-    $m add command -label [::msgcat::mc "Show info..."] \
+    $m add command -label [::msgcat::mc "Show info"] \
 	-command [list userinfo::open $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Show history..."] \
+    $m add command -label [::msgcat::mc "Show history"] \
 	-command [list logger::show_log $jid]
     $m add separator
-    $m add command -label [::msgcat::mc "Edit item..."] \
-	-command [list itemedit::show_dialog $connid $jid]
-    # TODO: connid
-    $m add command -label [::msgcat::mc "Edit security..."] \
-	-command [list ssj::prefs $jid] \
-	-state [lindex {normal disabled} \
-		    [cequal [info commands ::ssj::prefs] ""]]
+
+    hook::run roster_create_user_menu_edit_hook $m $connid $jid
+
     $m add separator
-    $m add command -label [::msgcat::mc "Remove..."] \
+    $m add command -label [::msgcat::mc "Remove item..."] \
 	-command [list [namespace current]::remove_item_dialog $connid $jid]
     return $m
 }
@@ -1503,6 +1586,7 @@ proc roster::service_popup_menu {connid jid} {
     $m add command -label [::msgcat::mc "Log out"] -command {
 	jlib::send_presence -to $curuser -type unavailable
     }
+    hook::run service_popup_menu_hook $m $connid $jid
     common_menu $m $connid $jid
 }
 
@@ -1524,15 +1608,22 @@ proc roster::create_groupchat_user_menu {connid jid} {
     $m add command -label [::msgcat::mc "Invite to conference..."] \
 	-command [list chat::invite_dialog $jid 0 -connection $connid]
     $m add separator
-    $m add command -label [::msgcat::mc "Show info..."] \
+    $m add command -label [::msgcat::mc "Show info"] \
 	-command [list userinfo::open $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Show history..."] \
+    $m add command -label [::msgcat::mc "Show history"] \
 	-command {} -state disabled
     return $m
 }
 
 proc roster::create_group_popup_menu {connid name} {
     variable options
+    variable chats_group_name
+
+    if {$name == $chats_group_name} {
+	set state disabled
+    } else {
+	set state normal
+    }
 
     if {[winfo exists [set m .grouppopupmenu]]} {
 	destroy $m
@@ -1543,8 +1634,9 @@ proc roster::create_group_popup_menu {connid name} {
 	set oname $name
     }
     menu $m -tearoff 0
-    $m add command -label [::msgcat::mc "Rename..."] \
-	-command [list [namespace current]::rename_group_dialog $connid $name]
+    $m add command -label [::msgcat::mc "Rename group..."] \
+	-command [list [namespace current]::rename_group_dialog $connid $name] \
+	-state $state
     $m add command \
 	-label [::msgcat::mc "Send message to all users in group..."] \
 	-command [list ::message::send_dialog \
@@ -1555,8 +1647,11 @@ proc roster::create_group_popup_menu {connid name} {
     $m add checkbutton -label [::msgcat::mc "Show offline users"] \
 	-variable [namespace current]::roster(show_offline,[list $connid $oname]) \
 	-command [list [namespace current]::redraw_after_idle]
-    $m add command -label [::msgcat::mc "Remove..."] \
-	-command [list [namespace current]::remove_group_dialog $connid $name]
+    $m add command -label [::msgcat::mc "Remove group..."] \
+	-command [list [namespace current]::remove_group_dialog $connid $name] \
+	-state $state
+    $m add command -label [::msgcat::mc "Remove all users in group..."] \
+	-command [list [namespace current]::remove_users_group_dialog $connid $name]
     return $m
 }
 
@@ -1565,13 +1660,25 @@ proc roster::create_group_popup_menu {connid name} {
 proc roster::remove_group_dialog {connid name} {
     set res [MessageDlg .remove_item -aspect 50000 -icon question -type user \
 		 -buttons {yes no} -default 0 -cancel 1 \
-		 -message [format [::msgcat::mc "Are you sure to remove group '%s' from roster?"] $name]]
+		 -message [format [::msgcat::mc "Are you sure to remove group '%s' from roster?\
+\n(Users which are in this group only, will be in undefined group.)"] $name]]
 
     if {$res == 0} {
 	roster::send_rename_group $connid $name ""
     }
 }
 
+proc roster::remove_users_group_dialog {connid name} {
+    set res [MessageDlg .remove_item -aspect 50000 -icon question -type user \
+		 -buttons {yes no} -default 0 -cancel 1 \
+		 -message [format [::msgcat::mc "Are you sure to remove all users in group '%s' from roster?\
+\n(Users which are in another groups too, will not be removed from the roster.)"] $name]]
+
+    if {$res == 0} {
+	roster::send_remove_users_group $connid $name
+    }
+}
+
 proc roster::rename_group_dialog {connid name} {
     global new_roster_group_name
 
@@ -1585,10 +1692,8 @@ proc roster::rename_group_dialog {connid name} {
     Dialog $w -title [::msgcat::mc "Rename roster group"] \
 	-separator 1 -anchor e -default 0 -cancel 1
 
-    $w add -text [::msgcat::mc "OK"] -command "
-	destroy [list $w]
-	roster::send_rename_group [list $connid $name] \$new_roster_group_name
-    "
+    $w add -text [::msgcat::mc "OK"] -command \
+	[list [namespace current]::confirm_rename_group $w $connid $name]
     $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
 
     set p [$w getframe]
@@ -1603,6 +1708,27 @@ proc roster::rename_group_dialog {connid name} {
     $w draw
 }
 
+proc roster::confirm_rename_group {w connid name} {
+    global new_roster_group_name
+    variable roster
+
+    destroy $w
+
+    ::roster::send_rename_group $connid $name $new_roster_group_name
+
+    set gid [list $connid $name]
+    set newgid [list $connid $new_roster_group_name]
+
+    if {[info exists roster(collapsed,$gid)]} {
+	set roster(collapsed,$newgid) $roster(collapsed,$gid)
+	unset roster(collapsed,$gid)
+    }
+    if {[info exists roster(show_offline,$gid)]} {
+	set roster(show_offline,$newgid) $roster(show_offline,$gid)
+	unset roster(show_offline,$gid)
+    }
+}
+	
 proc roster::add_group_by_jid_regexp_dialog {} {
     global new_roster_group_rname
     global new_roster_group_regexp
@@ -1661,7 +1787,8 @@ proc roster::send_users_dialog {user args} {
     catch { destroy $gw }
 
     if {[catch { set nick [::roster::get_label $connid $user] }]} {
-	if {[catch { set nick [chat::get_nick $user groupchat] }]} {
+	if {[catch { set nick [chat::get_nick $connid \
+					      $user groupchat] }]} {
 	    set nick $user
 	}
     }
@@ -1733,7 +1860,7 @@ hook::add finload_hook [namespace current]::roster::setup_import_export_menus
 proc roster::add_custom_presence_menu {m connid jids} {
     set mm [menu $m.custom_presence -tearoff 0]
 
-    add_menu_item $mm [::msgcat::mc "Online"] \
+    add_menu_item $mm [::msgcat::mc "Available"] \
 	"send_custom_presence \$curuser available -connection $connid" $jids
     add_menu_item $mm [::msgcat::mc "Free to chat"] \
 	"send_custom_presence \$curuser chat -connection $connid" $jids
@@ -1743,7 +1870,7 @@ proc roster::add_custom_presence_menu {m connid jids} {
 	"send_custom_presence \$curuser xa -connection $connid" $jids
     add_menu_item $mm [::msgcat::mc "Do not disturb"] \
 	"send_custom_presence \$curuser dnd -connection $connid" $jids
-    add_menu_item $mm [::msgcat::mc "Offline"] \
+    add_menu_item $mm [::msgcat::mc "Unavailable"] \
 	"send_custom_presence \$curuser unavailable -connection $connid" $jids
 
     $m add cascad -label [::msgcat::mc "Send custom presence"] -menu $mm
@@ -1751,24 +1878,3 @@ proc roster::add_custom_presence_menu {m connid jids} {
 
 ###############################################################################
 
-set ::roster_main_menu \
-    [list cascad [::msgcat::mc "Roster"] {} roster $ifacetk::options(show_tearoffs) \
-	 [list \
-	      [list command [::msgcat::mc "Add user..."] {} {} {} \
-		   -command {message::send_subscribe_dialog ""}] \
-	      [list command [::msgcat::mc "Add conference..."] {} {} {} \
-		   -command {add_group_dialog}] \
-	      [list command [::msgcat::mc "Add group by regexp on JIDs..."] {} {} {} \
-		   -command [namespace current]::roster::add_group_by_jid_regexp_dialog] \
-	      [list checkbutton [::msgcat::mc "Show online users only"] \
-		   {} {} {} \
-		   -variable [namespace current]::roster::show_only_online \
-		   -command [namespace current]::roster::redraw_after_idle] \
-	      [list checkbutton [::msgcat::mc "Use aliases"] {} {} {} \
-		   -variable [namespace current]::roster::use_aliases] \
-	      [list cascad [::msgcat::mc "Export roster..."] \
-		   export_roster export_roster 0 {}] \
-	      [list cascad [::msgcat::mc "Import roster..."] \
-		   import_roster import_roster 0 {}] \
-	     ]]
-
