@@ -1,27 +1,37 @@
-# $Id: login.tcl,v 1.63 2004/04/12 16:22:17 aleksey Exp $
+# $Id: login.tcl,v 1.77 2005/08/19 20:13:19 aleksey Exp $
 
 
-if {![info exists use_tls]} {
+if {[lcontain [jlib::capabilities transport] tls]} {
     set use_tls 1
-}
-if {$use_tls && [catch { package require tls 1.4 }]} {
-    debugmsg login "unable to load the TLS package, so no SSL support!
-
-The TLS package is available at http://tls.sf.net/"
-
+} else {
     set use_tls 0
 }
-if {!$use_tls} {
-    set loginconf(usessl) 0
+
+if {[lcontain [jlib::capabilities auth] sasl]} {
+    set have_sasl 1
+} else {
+    set have_sasl 0
 }
 
-set loginconf(usesasl) $jlib::lib(have_sasl)
+if {[lcontain [jlib::capabilities proxy] http]} {
+    set have_proxy 1
+} else {
+    set have_proxy 0
+}
+
+if {[lcontain [jlib::capabilities transport] http_poll]} {
+    set have_http_poll 1
+} else {
+    set have_http_poll 0
+}
 
 custom::defgroup Warnings [::msgcat::mc "Warning display options."] \
     -group Tkabber
 
-custom::defvar tls_warnings 1 [::msgcat::mc "Display SSL warnings."] \
-    -group Warnings -type boolean
+if {$use_tls} {
+    custom::defvar tls_warnings 1 [::msgcat::mc "Display SSL warnings."] \
+	-group Warnings -type boolean
+}
 
 custom::defgroup Login \
     [::msgcat::mc "Login options."] \
@@ -51,36 +61,50 @@ custom::defvar loginconf(priority) "8" \
 custom::defvar loginconf(connect_forever) 0 \
     [::msgcat::mc "Retry to connect forever."] \
     -group Login -type boolean
-custom::defvar loginconf(usessl) 0 \
-    [::msgcat::mc "Use SSL to connect to server."] \
-    -group Login -type boolean
-custom::defvar loginconf(sslcertfile) "" \
-    [::msgcat::mc "SSL certificate file (optional)."] \
-    -group Login -type string
-custom::defvar loginconf(sslcafile) "" \
-    [::msgcat::mc "SSL CA file (optional)."] \
-    -group Login -type string
-#custom::defvar loginconf(sslkeyfile) "" \
-#    [::msgcat::mc "SSL private key file (optional)."] \
-#    -group Login -type string
-custom::defvar loginconf(sslport) "5223" \
-    [::msgcat::mc "SSL port."] \
-    -group Login -type integer
-custom::defvar loginconf(useproxy) 0 \
-    [::msgcat::mc "Use HTTP proxy to connect."] \
-    -group Login -type boolean
-custom::defvar loginconf(httpproxy) "localhost" \
-    [::msgcat::mc "HTTP proxy address."] \
-    -group Login -type string
-custom::defvar loginconf(httpproxyport) 3128 \
-    [::msgcat::mc "HTTP proxy port."] \
-    -group Login -type integer
-custom::defvar loginconf(httplogin) "" \
-    [::msgcat::mc "HTTP proxy username."] \
-    -group Login -type string
-custom::defvar loginconf(httppassword) "" \
-    [::msgcat::mc "HTTP proxy password."] \
-    -group Login -type password
+
+if {$use_tls} {
+    custom::defvar loginconf(usessl) 0 \
+	[::msgcat::mc "SSL options when connecting to server."] \
+	-group Login -type radio -layout horizontal \
+	-values [list 0 [::msgcat::mc "Don't use SSL"] \
+		      1 [::msgcat::mc "Use legacy SSL"] \
+		      2 [::msgcat::mc "Use STARTTLS"]]
+    custom::defvar loginconf(sslport) "5223" \
+	[::msgcat::mc "SSL port."] \
+	-group Login -type integer
+    custom::defvar loginconf(sslcertfile) "" \
+	[::msgcat::mc "SSL certificate file (optional)."] \
+	-group Login -type file
+    custom::defvar loginconf(sslcafile) "" \
+	[::msgcat::mc "SSL CA file (optional)."] \
+	-group Login -type file
+    custom::defvar loginconf(sslkeyfile) "" \
+	[::msgcat::mc "SSL private key file (optional)."] \
+	-group Login -type file
+}
+
+if {$have_proxy} {
+    custom::defvar loginconf(useproxy) 0 \
+	[::msgcat::mc "Use HTTP proxy to connect."] \
+	-group Login -type boolean
+    custom::defvar loginconf(httpproxy) "localhost" \
+	[::msgcat::mc "HTTP proxy address."] \
+	-group Login -type string
+    custom::defvar loginconf(httpproxyport) 3128 \
+	[::msgcat::mc "HTTP proxy port."] \
+	-group Login -type integer
+    custom::defvar loginconf(httplogin) "" \
+	[::msgcat::mc "HTTP proxy username."] \
+	-group Login -type string
+    custom::defvar loginconf(httppassword) "" \
+	[::msgcat::mc "HTTP proxy password."] \
+	-group Login -type password
+    custom::defvar loginconf(httpuseragent) \
+	"Mozilla/4.0 (compatible; MSIE 6.0;\
+$::tcl_platform(os) $::tcl_platform(osVersion))" \
+	[::msgcat::mc "User-Agent string."] -group Login -type string
+}
+
 custom::defvar loginconf(usealtserver) 0 \
     [::msgcat::mc "Use explicitly-specified server address."] \
     -group Login -type boolean
@@ -90,36 +114,46 @@ custom::defvar loginconf(altserver) "" \
 custom::defvar loginconf(replace_opened) 1 \
     [::msgcat::mc "Replace opened connections."] \
     -group Login -type boolean
-custom::defvar loginconf(usehttppoll) 0 \
-    [::msgcat::mc "Use HTTP poll connection method."] \
-    -group Login -type boolean
-custom::defvar loginconf(pollurl) "http://connect.jabber.cz/" \
-    [::msgcat::mc "URL to connect to."] \
-    -group Login -type string
-custom::defvar loginconf(usepollkeys) 1 \
-    [::msgcat::mc "Use HTTP poll client security keys (recommended)."] \
-    -group Login -type boolean
-custom::defvar loginconf(numberofpollkeys) 100 \
-    [::msgcat::mc "Number of HTTP poll client security keys to send before creating new key sequence."] \
-    -group Login -type integer
-custom::defvar loginconf(polltimeout) 0 \
-    [::msgcat::mc "Timeout for waiting for HTTP poll responces (if set to zero, Tkabber will wait forever)."] \
-    -group Login -type integer
-custom::defvar loginconf(pollmin) 3000 \
-    [::msgcat::mc "Minimum poll interval."] \
-    -group Login -type integer
-custom::defvar loginconf(pollmax) 30000 \
-    [::msgcat::mc "Maximum poll interval."] \
-    -group Login -type integer
-custom::defvar loginconf(usesasl) $loginconf(usesasl) \
-    [::msgcat::mc "Use SASL authentification."] \
-    -group Login -type boolean
+
+if {$have_http_poll} {
+    custom::defvar loginconf(usehttppoll) 0 \
+	[::msgcat::mc "Use HTTP poll connection method."] \
+	-group Login -type boolean
+    custom::defvar loginconf(pollurl) "http://connect.jabber.cz/" \
+	[::msgcat::mc "URL to connect to."] \
+	-group Login -type string
+    custom::defvar loginconf(usepollkeys) 1 \
+	[::msgcat::mc "Use HTTP poll client security keys (recommended)."] \
+	-group Login -type boolean
+    custom::defvar loginconf(numberofpollkeys) 100 \
+	[::msgcat::mc "Number of HTTP poll client security keys to send\
+before creating new key sequence."] \
+	-group Login -type integer
+    custom::defvar loginconf(polltimeout) 0 \
+	[::msgcat::mc "Timeout for waiting for HTTP poll responces (if set\
+to zero, Tkabber will wait forever)."] \
+	-group Login -type integer
+    custom::defvar loginconf(pollmin) 3000 \
+	[::msgcat::mc "Minimum poll interval."] \
+	-group Login -type integer
+    custom::defvar loginconf(pollmax) 30000 \
+	[::msgcat::mc "Maximum poll interval."] \
+	-group Login -type integer
+}
+
+if {$have_sasl} {
+    custom::defvar loginconf(usesasl) 1 \
+	[::msgcat::mc "Use SASL authentication."] \
+	-group Login -type boolean
+    custom::defvar loginconf(allowsaslplain) 0 \
+	[::msgcat::mc "Allow plaintext SASL authentication mechanisms."] \
+	-group Login -type boolean
+}
 
 custom::defvar reasonlist {} [::msgcat::mc "List of logout reasons."] \
 	-group Hidden
 
-package require http
-
+######################################################################
 proc login {logindata} {
     global login_after_time
     global login_after_id
@@ -137,7 +171,7 @@ proc login {logindata} {
 	if {$lc(connect_forever)} {
 	    login_retry $logindata
 	} else {
-	    set res [MessageDlg .connect_err -aspect 50000 -icon error \
+	    set res [MessageDlg .connect_err -width 600 -icon error \
 		-message [format [::msgcat::mc "Failed to connect: %s"] $connid] \
 		-type user -buttons [list abort [::msgcat::mc "Keep trying"]] \
 		-default 0 -cancel 0]
@@ -174,14 +208,14 @@ proc login_retry {logindata} {
     set login_after_id($user) [after $login_after_time [list login $logindata]]
 }
 
-proc tls_callback {sock args} {
+proc client:tls_callback {connid args} {
     global tls_result tls_warnings
     global ssl_certificate_fields
     global tls_warning_info
 
     switch -- [lindex $args 0] {
 	info {
-	    set_status [join [lrange $args 2 end] " "]
+	    set_status [lindex $args 4]
 	}
 
 	verify {
@@ -189,7 +223,7 @@ proc tls_callback {sock args} {
 		return 1
 	    }
 	    set info [::msgcat::mc [string totitle $reason 0 0]]
-	    append tls_warning_info($sock) "$info\n"
+	    append tls_warning_info($connid) "$info\n"
 	    if {!$tls_warnings} {
 		return 1
 	    }
@@ -230,134 +264,87 @@ proc tls_callback {sock args} {
     }
 }
 
-proc tls_handshake {fd} {
-    global tls_count
-    global tls_failures
-    global tls_result
-
-    incr tls_count
-    if {[eof $fd]} {
-	set tls_result "EOF during TLS handshake"
-	fileevent $fd readable {}
-	fileevent $fd writable {}
-	return
-    }
-    flush $fd
-    set thrown [catch { tls::handshake $fd } shook]
-    if {(!$thrown) && ($shook)} {
-	set tls_result ""
-    } elseif {[incr tls_failures] > 15} {
-	set tls_result $shook
-    }
-}
-
 proc login_connect {logindata} {
-    global use_tls
+    global use_tls have_sasl have_http_poll have_proxy
     global tls_warning_info
 
     array set lc $logindata
-    if {!$lc(usehttppoll)} {
-	if {!$lc(useproxy)} {
-	    if {$lc(usealtserver)} {
-		set server $lc(altserver)
-	    } else {
-		set server $lc(server)
-	    }
-	    set server [jlib::idna_domain_toascii $server]
-	    if {$use_tls && $lc(usessl)} {
-		global tls_count
-		global tls_failures
-		global tls_result
 
-	        set sock [socket $server $lc(sslport)]
-		set tls_warning_info($sock) {}
-		set args [list -command [list [namespace current]::tls_callback $sock] \
-			       -ssl2    false                                          \
-			       -ssl3    true                                           \
-			       -tls1    true                                           \
-			       -request true                                           \
-			       -require false                                          \
-			       -server  false]
-		if {![cequal $lc(sslcertfile) ""]} {
-		    if {[cequal $lc(sslcafile) ""]} {
-		        lappend args -cafile $lc(sslcertfile)
-		    } else {
-		      lappend args -cafile $lc(sslcafile)
-		    }
-		}
-		eval [list tls::import $sock] $args
+    if {$lc(replace_opened) && [jlib::connections] != {}} {
+	return -1
+    }
 
-		fconfigure $sock -encoding binary -translation binary
+    set connid [jlib::new -user $lc(user) \
+			  -server $lc(server) \
+			  -resource $lc(resource)]
 
-		fileevent $sock readable \
-			  [list [namespace current]::tls_handshake $sock]
-		fileevent $sock writable {}
+    set tls_warning_info($connid) ""
 
-		set tls_count 0
-		set tls_failures 0
-		catch { unset tls_result }
-		tls_handshake $sock
-		while {![info exists tls_result]} {
-		    vwait tls_count
-		}
+    set args {}
+    if {$have_sasl} {
+	lappend args -usesasl $lc(usesasl) \
+		     -allowsaslplain $lc(allowsaslplain)
+    }
+    if {$have_proxy && $lc(useproxy)} {
+	lappend args -proxytype http
+	lappend args -proxyhost $lc(httpproxy)
+	lappend args -proxyport $lc(httpproxyport)
+	lappend args -proxyusername $lc(httplogin)
+	lappend args -proxypassword $lc(httppassword)
+	lappend args -proxyuseragent $lc(httpuseragent)
+    }
 
-		fileevent $sock readable {}
-
-		if {![cequal $tls_result ""]} {
-		    catch { close $sock }
-		    error $tls_result
-		}
-	    } else {
-	        set sock [socket $server $lc(port)]
-	    }
-	} else {
-	    set sock [connect_httpproxy $logindata]
-	}
-
-	return [jlib::connect $sock $lc(server) \
-			-newconnection [expr {!$lc(replace_opened)}] \
-			-user $lc(user) \
-			-resource $lc(resource) \
-			-usesasl $lc(usesasl)]
+    if {$have_http_poll && $lc(usehttppoll)} {
+	return [eval [list jlib::connect $connid \
+				-transport http_poll \
+				-polltimeout $lc(polltimeout) \
+				-pollint $lc(pollmin) \
+				-pollmin $lc(pollmin) \
+				-pollmax $lc(pollmax) \
+				-pollurl $lc(pollurl) \
+				-pollusekeys $lc(usepollkeys) \
+				-pollnumkeys $lc(numberofpollkeys)] $args]
     } else {
-	if $lc(useproxy) {
-	    set Proxy(use)      1
-	    ::http::config -proxyhost $lc(httpproxy) -proxyport $lc(httpproxyport)
-
-	    if {$lc(httplogin) != ""} {
-		set auth [base64::encode \
-                              [encoding convertto "$lc(httplogin):$lc(httppassword)"]]
-		set Proxy(auth) [list "Proxy-Authorization" "Basic $auth"]
-	    } else {
-		set Proxy(auth) {}
-	    }
+	if {$lc(usealtserver)} {
+	    set host $lc(altserver)
 	} else {
-	    set Proxy(use)      0
-	    set Proxy(auth)     {}
+	    set host $lc(server)
 	}
-	
-	return [jlib::connect {} $lc(server) \
-			-newconnection [expr {!$lc(replace_opened)}] \
-			-user $lc(user) \
-			-resource $lc(resource) \
-			-httppoll 1 \
-			-polltimeout $lc(polltimeout) \
-			-pollint $lc(pollmin) \
-			-pollmin $lc(pollmin) \
-			-pollmax $lc(pollmax) \
-			-pollurl $lc(pollurl) \
-			-pollusekeys $lc(usepollkeys) \
-			-pollnumkeys $lc(numberofpollkeys) \
-			-proxy $Proxy(use) \
-			-proxyauth $Proxy(auth) \
-			-usesasl $lc(usesasl)]
+
+	set transport tcp
+	set port $lc(port)
+	set sslopts {}
+	if {$use_tls} {
+	    switch -- $lc(usessl) {
+		1 {
+		    set usestarttls 0
+		    set transport tls
+		    set port $lc(sslport)
+		}
+		2 {
+		    set usestarttls 1
+		}
+		default {
+		    set usestarttls 0
+		}
+	    }
+	    set sslopts [list -usestarttls $usestarttls \
+	                      -certfile $lc(sslcertfile) \
+			      -cafile $lc(sslcafile) \
+			      -keyfile $lc(sslkeyfile)]
+	}
+
+	return [eval [list jlib::connect $connid \
+				-transport $transport \
+				-host $host \
+				-port $port] \
+				$sslopts $args]
     }
 }
-    
+
 proc login_login {logindata connid} {
     global loginconf_hist_$connid
     global gr_nick gr_server gra_server
-    global auth_result
 
     array set lc $logindata
     array set loginconf_hist_$connid $logindata
@@ -379,7 +366,7 @@ proc login_login {logindata connid} {
     switch -- $res {
 	ERR {
 	    clear_status
-	    recv_auth_result $connid ERR $data
+	    recv_auth_result $connid $logindata ERR $data
 	    return
 	}
 	OK {
@@ -413,14 +400,9 @@ support\nhashed password authentication.\n\nProceed with PLAINTEXT authenticatio
 	}
     }
 
-    jlib::send_auth \
+    jlib::send_auth $connid \
 	$lc(user) $lc(password) $lc(resource) \
-	[list recv_auth_result $connid] $authtype $connid
-    vwait auth_result($connid)
-
-    if {$auth_result($connid) == "OK"} {
-        connected $logindata $connid
-    }
+	[list recv_auth_result $connid $logindata] $authtype
 }
 
 set reconnect_retries 0
@@ -435,6 +417,8 @@ proc logout {{connid {}}} {
 	unset login_after_id($user)
     }
 
+    hook::run predisconnected_hook $connid
+
     jlib::disconnect $connid
     if {$connid == {}} {
 	roster::clean
@@ -485,83 +469,38 @@ proc disconnected {connid} {
     hook::run disconnected_hook $connid
 }
 
-package require base64
-
-proc connect_httpproxy {logindata} {
-    global use_tls
-
-    array set lc $logindata
-    set sock [socket $lc(httpproxy) $lc(httpproxyport)]
-    fconfigure $sock -buffering line
-
-    if {$lc(usealtserver)} {
-	set server $lc(altserver)
-    } else {
-	set server $lc(server)
-    }
-    set server [jlib::idna_domain_toascii $server]
-    if {$use_tls && $lc(usessl)} {
-	puts $sock \
-	    "CONNECT $server:${lc(sslport)} HTTP/1.0"
-    } else {
-	puts $sock "CONNECT $server:${lc(port)} HTTP/1.0"
-    }
-    
-    if {$lc(httplogin) != ""} {
-	set auth [base64::encode \
-		      [encoding convertto "$lc(httplogin):$lc(httppassword)"]]
-	puts $sock "Proxy-Authorization: Basic $auth"
-    }
-    puts $sock ""
-
-    fileevent $sock readable {set proxy_readable ""}
-    global proxy_readable
-    vwait proxy_readable
-    fileevent $sock readable {}
-
-    set result [gets $sock]
-
-    set code [lindex [split $result { }] 1]
-
-    #debugmsg login $code
-    if {$code >= 200 && $code < 300} {
-	while {![cequal [gets $sock] ""]} { }
-	if {$use_tls && $lc(usessl)} {
-	    if {[cequal $lc(sslcertfile) ""]} {
-		tls::import $sock
-	    } else {
-		tls::import $sock -cafile $lc(sslcafile)
-	    }
-	}
-	return $sock
-    } else {
-	error "proxy return: $result"
-    }
-}
-
-proc recv_auth_result {connid res args} {
-    global auth_result
+proc recv_auth_result {connid logindata res args} {
     upvar #0 loginconf_hist_$connid lc
 
     if {$res == "OK"} {
-	set auth_result($connid) OK
+	connected $logindata $connid
     } else {
-	set auth_result($connid) ERR
-	set res [MessageDlg .auth_err -aspect 50000 -icon error \
-		     -message [format \
-				   [::msgcat::mc "Authentication failed: %s\nCreate new account?"] \
-				   [error_to_string [lindex $args 0]]] \
-		     -type user -buttons {yes no} -default 0 -cancel 1]
-	if {!$res} {
-	    jlib::send_iq set \
-		[jlib::wrapper:createtag query \
-		     -vars {xmlns jabber:iq:register} \
-		     -subtags [list [jlib::wrapper:createtag username \
-					 -chdata $lc(user)] \
-				   [jlib::wrapper:createtag password \
-					-chdata $lc(password)]]] \
-		-command [list recv_register_result [array get lc]]
+	lassign [error_type_condition [lindex $args 0]] type cond
+	if {($type == "sasl") || ($type == "auth" && $cond == "not-authorized")} {
+	    set res [MessageDlg .auth_err -aspect 50000 -icon error \
+			 -message [format \
+			     [::msgcat::mc "Authentication failed: %s\nCreate new account?"] \
+			     [error_to_string [lindex $args 0]]] \
+			 -type user -buttons {yes no} -default 0 -cancel 1]
+	    if {!$res} {
+		jlib::send_iq set \
+		    [jlib::wrapper:createtag query \
+			 -vars {xmlns jabber:iq:register} \
+			 -subtags [list [jlib::wrapper:createtag username \
+					     -chdata $lc(user)] \
+					[jlib::wrapper:createtag password \
+					     -chdata $lc(password)]]] \
+		    -command [list recv_register_result [array get lc]]
+		return
+	    }
+	} else {
+	    MessageDlg .auth_err -aspect 50000 -icon error \
+		-message [format \
+			     [::msgcat::mc "Authentication failed: %s"] \
+			     [error_to_string [lindex $args 0]]] \
+		-type user -buttons {ok} -default 0 -cancel 0
 	}
+	logout $connid
     }
 }
 
@@ -574,6 +513,7 @@ proc recv_register_result {logindata res args} {
 	    -message [format [::msgcat::mc "Registration failed: %s"] \
 			  [error_to_string [lindex $args 0]]] \
 	    -type user -buttons ok -default 0 -cancel 0
+	logout $connid
     }
 }
 
@@ -717,7 +657,7 @@ proc show_logout_dialog {} {
 }
 
 proc logout_reason {} {
-    global userstatus textstatus reason reasonlist
+    global logoutuserstatus logouttextstatus logoutpriority reason reasonlist
 
     set reasonlist [update_combo_list $reasonlist $reason 10]
     set custom::saved(::::reasonlist) $reasonlist
@@ -726,8 +666,10 @@ proc logout_reason {} {
     set lw .logout
     $lw withdraw
 
-    set textstatus $reason
-    set userstatus unavailable
+    # TODO
+    set logoutpriority $::loginconf(priority)
+    set logouttextstatus $reason
+    set logoutuserstatus unavailable
 
     logout
 
