@@ -1,4 +1,4 @@
-# $Id: messages.tcl,v 1.79 2004/07/10 20:05:04 aleksey Exp $
+# $Id: messages.tcl,v 1.89 2005/08/06 22:33:12 aleksey Exp $
 
 namespace eval message {
     variable msgid 0
@@ -48,6 +48,7 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
 
     set mw .msg$msgid
     toplevel $mw -class Message
+    wm group $mw .
 
     if {$replyP} {
         set title [format [::msgcat::mc "Message from %s"] $from]
@@ -72,13 +73,13 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
             "
         }
         if {![cequal [info commands ::ssj::encrypt:toggleP] ""]} {
-            $bbox1 add -image [message::encrypted:icon $from] \
+            $bbox1 add -image [message::encrypted:icon $connid $from] \
                     -helptype balloon -helptext [::msgcat::mc "Toggle encryption"] \
                     -height 24 -width 24 -relief link -bd $::tk_borderwidth \
-                    -command [list ssj::encrypt:toggleP $from]
+                    -command [list ssj::encrypt:toggleP $connid $from]
             ssj::encrypted:trace "
-                $mw.bottom.encrypted configure -image \[message::encrypted:icon $from\]
-            " $from
+                $mw.bottom.encrypted configure -image \[message::encrypted:icon $connid $from\]
+            " $connid $from
         }
         pack $bbox1 -side left -fill x -padx 2m -pady 2m
 
@@ -156,8 +157,6 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
 
     incr row
     set last $row
-    #set x [list [read_file /tmp/alex/xdata.tcl]]
-    #debugmsg message $x
     set namespaces {}
     foreach xe $x {
         lassign [process_x $mw.rf $xe $row $from $type $body $namespaces $replyP] row body namespaces
@@ -167,6 +166,11 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
         destroy $mw
         return
     }
+    # Ignore messages with empty body and no (or unsupported, e.g. jabber:x:event) extras
+    if {[cequal $body ""] && ($row == $last)} {
+        destroy $mw
+        return
+    }
 
     ScrolledWindow $mw.rsw
     text $mw.rbody -width 60 -height 8 -wrap word
@@ -210,7 +214,7 @@ proc message::show_dialog {connid from type subject body thread priority x {repl
         ScrolledWindow $mw.sw
         pack $mw.sw -in $mw.frame -side top -fill both -expand yes
 
-        textUndoable $mw.body -width 60 -height 8 -wrap word
+        textUndoable $mw.body -width 60 -height 8 -wrap word -font $font
         pack $mw.body -side top -fill both -expand yes -in $mw.sw
         bind $mw.body <Control-z> { %W undo }
         bind $mw.body <Control-Z> { %W redo }
@@ -245,8 +249,8 @@ proc message::process_x {f x row from type body namespaces replyP} {
             set row [process_x_oob $f $children $row $from]
         } \
         jabber:x:conference {
-            if {![lcontain $namespaces $::NS(gc_user)]} {
-                process_x_conference $f [jlib::wrapper:getattr $vars jid] $row
+            if {![lcontain $namespaces $::NS(muc#user)]} {
+                process_x_conference $f [jlib::wrapper:getattr $vars jid] "" $row
                 incr row
                 if {[cequal $body ""] && ![cequal $chdata ""]} {
                     set body $chdata
@@ -266,7 +270,9 @@ proc message::process_x {f x row from type body namespaces replyP} {
         jabber:x:data {
             process_x_data $f $from $x
         } \
-        $::NS(gc_user) {
+        $::NS(muc#user) {
+	    set password ""
+	    set inviter ""
             foreach ch $children {
                 jlib::wrapper:splitxml $ch tag1 vars1 isempty1 chdata1 children1
 
@@ -274,9 +280,6 @@ proc message::process_x {f x row from type body namespaces replyP} {
                     invite {
                         set inviter [jlib::wrapper:getattr $vars1 from]
                         if {![cequal $inviter ""]} {
-                            process_x_conference $f $from $row
-                            incr row
-
                             foreach c [jlib::connections] {
                                 set name [roster::itemconfig $c \
                                               [roster::find_jid $c $inviter] \
@@ -298,8 +301,15 @@ proc message::process_x {f x row from type body namespaces replyP} {
                         }
                     # TODO decline
                     }
+		    password {
+			set password $chdata1
+		    }
                 }
             }
+	    if {![string equal $inviter ""]} {
+		process_x_conference $f $from $password $row
+		incr row
+	    }
         }
 
     return [list $row $body $namespaces]
@@ -376,9 +386,12 @@ proc message::process_x_oob {f x row from} {
     return $row
 }
 
-proc message::process_x_conference {f conf row} {
+proc message::process_x_conference {f conf password row} {
     label $f.lconf$row -text [::msgcat::mc "Invited to:"]
-    button $f.conf$row -text $conf -command [list join_group $conf -nick [get_group_nick $conf $::gr_nick]]
+    button $f.conf$row -text $conf \
+	-command [list join_group $conf \
+		       -nick [get_group_nick $conf $::gr_nick] \
+		       -password $password]
     
     grid $f.lconf$row -row $row -column 0 -sticky e
     grid $f.conf$row  -row $row -column 1 -sticky ew
@@ -405,7 +418,7 @@ proc message::process_x_signed {f from signature data} {
     }
 
     ssj::signed:Label $lb \
-       [ssj::signed:input $from $signature $data [::msgcat::mc "Message body"]]
+       [ssj::signed:input $connid $from $signature $data [::msgcat::mc "Message body"]]
     pack $lb -side left
 }
 
@@ -436,6 +449,7 @@ proc message::send_x_data {to w restags} {
 }
 
 proc message::send_dialog {args} {
+    global font
     global messageicon
     variable msgid
     variable message_dest_list
@@ -468,7 +482,8 @@ proc message::send_dialog {args} {
     }
 
     set mw .msg$msgid
-    toplevel $mw
+    toplevel $mw -class Message
+    wm group $mw .
 
     if {$group} {
 	if {$to != ""} {
@@ -509,13 +524,13 @@ proc message::send_dialog {args} {
     }
     if {![cequal [info commands ::ssj::encrypt:toggleP] ""]} {
         set b $bbox1.b[llength [winfo children $bbox1]]
-        $bbox1 add -image [message::encrypted:icon $to] \
+        $bbox1 add -image [message::encrypted:icon $connid $to] \
                -helptype balloon -helptext [::msgcat::mc "Toggle encryption"] \
                -height 24 -width 24 -relief link -bd $::tk_borderwidth \
-               -command [list eval ssj::encrypt:toggleP $to]
+               -command [list eval ssj::encrypt:toggleP $connid $to]
         ssj::encrypted:trace "
-            $b configure -image \[message::encrypted:icon $to\]
-        " $to
+            $b configure -image \[message::encrypted:icon $connid $to\]
+        " $connid $to
     }
     pack $bbox1 -side left -fill x -padx 2m -pady 2m
 
@@ -569,7 +584,7 @@ proc message::send_dialog {args} {
     ScrolledWindow $mw.sw
     pack $mw.sw -in $mw.frame -side top -fill both -expand yes
 
-    textUndoable $mw.body -width 60 -height 8 -wrap word
+    textUndoable $mw.body -width 60 -height 8 -wrap word -font $font
     pack $mw.body -side top -fill both -expand yes -in $mw.sw
     bind $mw.body <Control-z> { %W undo }
     bind $mw.body <Control-Z> { %W redo }
@@ -646,7 +661,7 @@ proc message::send_msg {to args} {
                 continue
             }
 
-            if {[catch { ssj::${tag}:output $params(-body) $to } chdata]} {
+            if {[catch { ssj::${tag}:output $params(-connection) $params(-body) $to } chdata]} {
                 debugmsg message "ssj::${tag}:output: $chdata"
                 return [list error ssj]
             }
@@ -710,6 +725,7 @@ proc message::show_subscribe_dialog {from x args} {
 
     set mw .msg$msgid
     toplevel $mw -class Message
+    wm group $mw .
 
     set title [format [::msgcat::mc "Subscribe request from %s"] $from]
     wm title $mw $title
@@ -807,9 +823,9 @@ proc message::subject_menu {m jid type args} {
         $m add command -label [::msgcat::mc "Add user..."] \
             -command [list message::send_subscribe_dialog $jid -connection $connid]
     }
-    $m add command -label [::msgcat::mc "Show info..."] \
+    $m add command -label [::msgcat::mc "Show info"] \
         -command [list userinfo::open $jid -connection $connid]
-    $m add command -label [::msgcat::mc "Show history..."] -command {} -state disabled
+    $m add command -label [::msgcat::mc "Show history"] -command {} -state disabled
 }   
 
 proc message::send_subscribe_dialog {to args} {
@@ -825,7 +841,8 @@ proc message::send_subscribe_dialog {to args} {
     }
 
     set mw .msg$msgid
-    toplevel $mw
+    toplevel $mw -class Message
+    wm group $mw .
 
     if {[cequal $to ""]} {
         set title [::msgcat::mc "Send subscription"]
@@ -1076,7 +1093,7 @@ proc message::update_headlines_menu {menu num} {
     foreach jid $headline_send_jids {
 	$menu insert $ind command \
 	    -label [format [::msgcat::mc "Forward to %s"] $jid] \
-	    -command "message::headline_forward3 $menu $jid \
+	    -command "message::headline_forward3 [list $menu] [list $jid] \
 			  \$::message::headwindow \$::message::headnode"
 	incr ind
     }
@@ -1092,6 +1109,8 @@ $m add separator
 $m add command -label [::msgcat::mc "Forward..."] \
       -command {message::headline_action forward $::message::headwindow $::message::headnode}
 $m add separator
+$m add command -label [::msgcat::mc "Copy headline to clipboard"] \
+      -command {message::headline_action copy_headline $::message::headwindow $::message::headnode}
 $m add command -label [::msgcat::mc "Copy URL to clipboard"] \
       -command {message::headline_action copy_url $::message::headwindow $::message::headnode}
 $m add command -label [::msgcat::mc "Toggle seen"] \
@@ -1166,6 +1185,11 @@ proc message::headline_action {action hw node} {
 	     message::headline_forward .h1popmenu $tw $node
 	}
 
+	article/copy_headline {
+	    clipboard clear -displayof $hw
+	    clipboard append -displayof $hw "$props(text)\n$props(body)\n$props(url)"
+	}
+
 	article/copy_url {
 	    clipboard clear -displayof $hw
 	    clipboard append -displayof $hw $props(url)
@@ -1389,10 +1413,10 @@ proc message::restore_headlines {} {
 	if {[file exists [set file [file join ~ .tkabber headlines.tcl]]]} {
 	    set encoding [encoding system]
 	    encoding system utf-8
-	    source $file
+	    catch { source $file }
 	    encoding system $encoding
 	} elseif {[file exists [set file [file join ~ .tkabber .headlines.tcl]]]} {
-	    source $file
+	    catch { source $file }
 	}
     }
 
@@ -1492,11 +1516,11 @@ proc message::signed:icon {} {
                    [ssj::signP]]
 }
 
-proc message::encrypted:icon {{arg ""}} {
+proc message::encrypted:icon {{connid ""} {jid ""}} {
     global messageicon
 
     return [lindex [list $messageicon(unencrypted) $messageicon(encrypted)] \
-                   [ssj::encryptP $arg]]
+                   [ssj::encryptP $connid $jid]]
 }
 
 hook::add finload_hook message::restore_headlines
