@@ -1,4 +1,4 @@
-# $Id: muc.tcl,v 1.59 2004/04/17 19:01:28 aleksey Exp $
+# $Id: muc.tcl,v 1.67 2005/08/06 22:13:32 aleksey Exp $
 
 # Multi-User Chat support (JEP-0045)
 
@@ -22,13 +22,17 @@ namespace eval muc {
 }
 
 
-set ::NS(muc)      http://jabber.org/protocol/muc
-set ::NS(gc_admin) http://jabber.org/protocol/muc#admin
-set ::NS(gc_owner) http://jabber.org/protocol/muc#owner
-set ::NS(gc_user)  http://jabber.org/protocol/muc#user
+set ::NS(muc)       http://jabber.org/protocol/muc
+set ::NS(muc#admin) http://jabber.org/protocol/muc#admin
+set ::NS(muc#owner) http://jabber.org/protocol/muc#owner
+set ::NS(muc#user)  http://jabber.org/protocol/muc#user
 
 
 proc muc::add_groupchat_user_menu_items {m connid jid} {
+    set group [node_and_server_from_jid $jid]
+
+    if {![is_compatible $group]} return
+
     set mm [menu $m.muc -tearoff 0]
     $mm add command -label [::msgcat::mc "Whois"] \
 	-command [list muc::whois $connid $jid]
@@ -75,11 +79,23 @@ hook::add roster_create_groupchat_user_menu_hook \
     muc::add_groupchat_user_menu_items
 
 
-proc muc::add_conference_menu_items {m chatid} {
+proc muc::create_conference_menu_items {m chatid} {
     set group [chat::get_jid $chatid]
 
-    wait_for_negotiation $group
-    if {![is_compatible $group]} return
+    trace variable ::muc::muc_compatible($group) w \
+	[list muc::add_conference_menu_items $m $chatid]
+}
+    
+hook::add chat_create_conference_menu_hook muc::create_conference_menu_items
+
+
+proc muc::add_conference_menu_items {m chatid args} {
+    set group [chat::get_jid $chatid]
+
+    if {![is_compatible $group] || ![winfo exists $m]} return
+
+    trace vdelete ::muc::muc_compatible($group) w \
+	[list muc::add_conference_menu_items $m $chatid]
 
     $m add command -label [::msgcat::mc "Configure room..."] \
 	-command [list muc::request_config query $chatid]
@@ -96,11 +112,9 @@ proc muc::add_conference_menu_items {m chatid} {
     $m add command -label [::msgcat::mc "Edit owner list..."] \
 	-command [list muc::request_list owner affiliation owner $chatid]
     $m add command -label [::msgcat::mc "Destroy room"] \
-	-command [list muc::request_destruction $chatid "" ""]
+	-command [list muc::request_destruction_dialog $chatid "" ""]
 }
 
-hook::add chat_create_conference_menu_hook muc::add_conference_menu_items
-
 
 proc muc::handle_commands {chatid user body type} {
     if {$type != "groupchat"} return
@@ -210,6 +224,7 @@ proc muc::commands_comps {chatid compsvar wordstart line} {
 
 hook::add generate_completions_hook muc::commands_comps
 
+
 proc muc::get_real_jid {connid jid} {
     variable users
 
@@ -224,7 +239,7 @@ proc muc::get_real_jid {connid jid} {
 proc muc::whois {connid user} {
     set group [node_and_server_from_jid $user]
     set chatid [chat::chatid $connid $group]
-    set nick [chat::get_nick $user groupchat]
+    set nick [chat::get_nick $connid $user groupchat]
 
     set real_jid [get_real_jid $connid $user]
 
@@ -238,7 +253,7 @@ proc muc::whois {connid user} {
 
 proc muc::change_item_param {level params connid user reason} {
     set group [node_and_server_from_jid $user]
-    set nick  [chat::get_nick $user groupchat]
+    set nick  [chat::get_nick $connid $user groupchat]
 
     set itemsubtags {}
     if {$reason != ""} {
@@ -251,7 +266,7 @@ proc muc::change_item_param {level params connid user reason} {
 
     jlib::send_iq set \
 	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $::NS(gc_$level)] \
+	     -vars [list xmlns $::NS(muc#$level)] \
 	     -subtags [list $item]] \
 	-to $group \
 	-connection $connid \
@@ -259,13 +274,30 @@ proc muc::change_item_param {level params connid user reason} {
 }
 
 
+proc muc::request_destruction_dialog {chatid alt reason} {
+    set connid [chat::get_connid $chatid]
+    set group [chat::get_jid $chatid]
+
+    set warning \
+	[format [::msgcat::mc "Conference room %s will be destroyed permanently.\n\nProceed?"] \
+		$group]
+
+    set res [MessageDlg .muc_request_destruction -aspect 50000 -icon warning \
+			-type user -buttons {yes no} -default 1 \
+			-cancel 1 \
+			-message $warning]
+    if {!$res} {
+	request_destruction $chatid $alt $reason
+    }
+}
+
 proc muc::request_destruction {chatid alt reason} {
     set connid [chat::get_connid $chatid]
     set group [chat::get_jid $chatid]
 
     jlib::send_iq set \
 	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $::NS(gc_owner)] \
+	     -vars [list xmlns $::NS(muc#owner)] \
 	     -subtags [list \
 			   [jlib::wrapper:createtag destroy \
 				-subtags [list \
@@ -284,7 +316,7 @@ proc muc::request_list {level attr val chatid} {
     set group [chat::get_jid $chatid]
     jlib::send_iq get \
 	[jlib::wrapper:createtag query \
-	     -vars [list xmlns $::NS(gc_$level)] \
+	     -vars [list xmlns $::NS(muc#$level)] \
 	     -subtags [list [jlib::wrapper:createtag item \
 				 -vars [list $attr $val]]]] \
 	-to $group \
@@ -328,13 +360,12 @@ proc muc::receive_list {level attr val chatid res child} {
     list_add_item $sf $f $attr $val
  
     $w add -text [::msgcat::mc "Send"] \
-	-command [list [namespace current]::send_list \
-		      $chatid $level $attr $val $w $f]
+	-command [list muc::send_list $chatid $level $attr $val $w $f]
     $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]
-    bind $w <Destroy> [list [namespace current]::list_cleanup $w $f]
+    bind $w <Destroy> [list muc::list_cleanup $w $f]
 
     button $w.add -text [::msgcat::mc "Add"] \
-	-command [list [namespace current]::list_add_item $sf $f $attr $val]
+	-command [list muc::list_add_item $sf $f $attr $val]
     pack $w.add -side bottom -anchor e -in $wf -padx 1m -pady 1m
     pack $sw -side top -expand yes -fill both
 
@@ -393,6 +424,7 @@ proc muc::fill_list {sf f items attr val} {
 
     set row 1
 
+    set items2 {}
     foreach item $items {
 	jlib::wrapper:splitxml $item tag vars isempty chdata children
 	switch -- $tag {
@@ -401,50 +433,53 @@ proc muc::fill_list {sf f items attr val} {
 		set jid [jlib::wrapper:getattr $vars jid]
 		set role [jlib::wrapper:getattr $vars role]
 		set affiliation [jlib::wrapper:getattr $vars affiliation]
-
-		label $f.nick$row -text $nick -font $font \
-		    -textvariable [namespace current]::listdata($f,nick,$row)
-		grid $f.nick$row -row $row -column 0 -sticky w -padx 1m
-		bindscroll $f.nick$row $sf
-
-		label $f.jid$row -text $jid -font $font \
-		    -textvariable [namespace current]::listdata($f,jid,$row)
-		grid $f.jid$row -row $row -column 1 -sticky w -padx 1m
-		bindscroll $f.jid$row $sf
-
-		switch -- $attr {
-		    role {
-			ComboBox $f.role$row -text $role \
-			    -values {moderator participant visitor none} \
-			    -editable no \
-			    -width 11 \
-			    -textvariable \
-			    [namespace current]::listdata($f,role,$row)
-			grid $f.role$row -row $row -column 2 -sticky we -padx 1m
-			bindscroll $f.role$row $sf
-		    }
-		    affiliation {
-			ComboBox $f.affiliation$row -text $affiliation \
-			    -values {owner admin member none outcast} \
-			    -editable no \
-			    -width 7 \
-			    -textvariable \
-			    [namespace current]::listdata($f,affiliation,$row)
-			grid $f.affiliation$row -row $row -column 2 -sticky we -padx 1m
-			bindscroll $f.affiliation$row $sf
-		    }
-		}
-
-		entry $f.reason$row -font $font \
-		    -textvariable [namespace current]::listdata($f,reason,$row)
-		grid $f.reason$row -row $row -column 3 -sticky we -padx 1m
-		bindscroll $f.reason$row $sf
-		
-		incr row
+		lappend items2 [list $nick $jid $role $affiliation]
 	    }
 	}
     }
 
+    foreach item [lsort -dictionary -index 1 $items2] {
+	lassign $item nick jid role affiliation
+
+	label $f.nick$row -text $nick -font $font \
+	    -textvariable muc::listdata($f,nick,$row)
+	grid $f.nick$row -row $row -column 0 -sticky w -padx 1m
+	bindscroll $f.nick$row $sf
+
+	label $f.jid$row -text $jid -font $font \
+	    -textvariable muc::listdata($f,jid,$row)
+	grid $f.jid$row -row $row -column 1 -sticky w -padx 1m
+	bindscroll $f.jid$row $sf
+
+	switch -- $attr {
+	    role {
+		ComboBox $f.role$row -text $role \
+		    -values {moderator participant visitor none} \
+		    -editable no \
+		    -width 11 \
+		    -textvariable muc::listdata($f,role,$row)
+		grid $f.role$row -row $row -column 2 -sticky we -padx 1m
+		bindscroll $f.role$row $sf
+	    }
+	    affiliation {
+		ComboBox $f.affiliation$row -text $affiliation \
+		    -values {owner admin member none outcast} \
+		    -editable no \
+		    -width 7 \
+		    -textvariable muc::listdata($f,affiliation,$row)
+		grid $f.affiliation$row -row $row -column 2 -sticky we -padx 1m
+		bindscroll $f.affiliation$row $sf
+	    }
+	}
+
+	entry $f.reason$row -font $font \
+	    -textvariable muc::listdata($f,reason,$row)
+	grid $f.reason$row -row $row -column 3 -sticky we -padx 1m
+	bindscroll $f.reason$row $sf
+	
+	incr row
+    }
+
     set listdata($f,rows) [incr row -1]
     array set origlistdata [array get listdata ${f}*]
 }
@@ -456,13 +491,11 @@ proc muc::list_add_item {sf f attr val} {
 
     set row [incr listdata($f,rows)]
 
-    entry $f.nick$row -font $font \
-	-textvariable [namespace current]::listdata($f,nick,$row)
+    entry $f.nick$row -font $font -textvariable muc::listdata($f,nick,$row)
     grid $f.nick$row -row $row -column 0 -sticky we -padx 1m
     bindscroll $f.nick$row $sf
     
-    entry $f.jid$row -font $font \
-	-textvariable [namespace current]::listdata($f,jid,$row)
+    entry $f.jid$row -font $font -textvariable muc::listdata($f,jid,$row)
     grid $f.jid$row -row $row -column 1 -sticky we -padx 1m
     bindscroll $f.jid$row $sf
 
@@ -472,7 +505,7 @@ proc muc::list_add_item {sf f attr val} {
 		-values {moderator participant visitor none} \
 		-editable no \
 		-width 11 \
-		-textvariable [namespace current]::listdata($f,role,$row)
+		-textvariable muc::listdata($f,role,$row)
 	    grid $f.role$row -row $row -column 2 -sticky we -padx 1m
 	    bindscroll $f.role$row $sf
 	}
@@ -481,15 +514,13 @@ proc muc::list_add_item {sf f attr val} {
 		-values {owner admin member none outcast} \
 		-editable no \
 		-width 7 \
-		-textvariable \
-		[namespace current]::listdata($f,affiliation,$row)
+		-textvariable muc::listdata($f,affiliation,$row)
 	    grid $f.affiliation$row -row $row -column 2 -sticky we -padx 1m
 	    bindscroll $f.affiliation$row $sf
 	}
     }
 
-    entry $f.reason$row -font $font \
-	-textvariable [namespace current]::listdata($f,reason,$row)
+    entry $f.reason$row -font $font -textvariable muc::listdata($f,reason,$row)
     grid $f.reason$row -row $row -column 3 -sticky we -padx 1m
     bindscroll $f.reason$row $sf
 
@@ -560,7 +591,7 @@ proc muc::send_list {chatid level attr val w f} {
 
     if {$items != {}} {
 	jlib::send_iq set [jlib::wrapper:createtag query \
-			       -vars [list xmlns $::NS(gc_$level)] \
+			       -vars [list xmlns $::NS(muc#$level)] \
 			       -subtags $items] \
 	    -to $group \
 	    -connection $connid \
@@ -582,7 +613,7 @@ proc muc::list_cleanup {w f} {
 proc muc::request_config {op chatid} {
     jlib::send_iq get \
 	[jlib::wrapper:createtag $op \
-	     -vars [list xmlns $::NS(gc_owner)]] \
+	     -vars [list xmlns $::NS(muc#owner)]] \
 	-to [chat::get_jid $chatid] \
 	-connection [chat::get_connid $chatid] \
 	-command [list muc::receive_config $op $chatid]
@@ -613,7 +644,7 @@ proc muc::send_config {op chatid w restags} {
     set connid [chat::get_connid $chatid]
     set group [chat::get_jid $chatid]
     jlib::send_iq set [jlib::wrapper:createtag $op \
-			   -vars [list xmlns $::NS(gc_owner)] \
+			   -vars [list xmlns $::NS(muc#owner)] \
 			   -subtags $restags] \
 	-to $group \
 	-connection $connid \
@@ -630,8 +661,18 @@ proc muc::test_error_res {op connid group res child} {
     }
 }
 
+proc muc::process_presence_x {connid from type xs} {
+    foreach x $xs {
+	jlib::wrapper:splitxml $x tag vars isempty chdata children
+	if {[jlib::wrapper:getattr $vars xmlns] == $::NS(muc#user)} {
+	    process_muc_user $connid $from $type $children
+	}
+    }
+}
 
-proc muc::process_gc_user {connid user type childrens} {
+hook::add presence_process_x_hook muc::process_presence_x
+
+proc muc::process_muc_user {connid user type childrens} {
     variable users
     variable options
 
@@ -667,7 +708,7 @@ proc muc::process_gc_user {connid user type childrens} {
 		    307 {
 			set group [node_and_server_from_jid $user]
 			set chatid [chat::chatid $connid $group]
-			set nick [chat::get_nick $user groupchat]
+			set nick [chat::get_nick $connid $user groupchat]
 			switch -- $code {
 			    301 {set action \
 				     [format \
@@ -691,6 +732,7 @@ proc muc::process_gc_user {connid user type childrens} {
 			}
 
 			if {$options(gen_events)} {
+			    variable ignore_unavailable $nick
 			    chat::add_message \
 				$chatid $group groupchat $action {}
 			}
@@ -698,7 +740,7 @@ proc muc::process_gc_user {connid user type childrens} {
 		    303 {
 			set group [node_and_server_from_jid $user]
 			set chatid [chat::chatid $connid $group]
-			set nick [chat::get_nick $user groupchat]
+			set nick [chat::get_nick $connid $user groupchat]
 			if {[info exists new_nick] && $new_nick != ""} {
 			    if {$nick == [get_our_groupchat_nick $chatid]} {
 				set_our_groupchat_nick $chatid $new_nick
@@ -789,7 +831,7 @@ proc muc::change_nick {chatid nick} {
 
     jlib::send_presence -to $group/$nick \
 	-connection [chat::get_connid $chatid] \
-	-command [list [namespace current]::process_change_nick $chatid]
+	-command [list muc::process_change_nick $chatid]
 }
 
 
@@ -829,7 +871,7 @@ proc muc::is_changing_nick {chatid} {
 proc muc::request_negotiation {connid group} {
     variable muc_compatible
 
-    catch { unset muc_compatible($group) }
+    set muc_compatible($group) 0
 
     disco::request_info [server_from_jid $group] "" \
 	-connection $connid \
@@ -838,27 +880,19 @@ proc muc::request_negotiation {connid group} {
 }
 
 
-proc muc::recv_negotiation {group res identities features} {
+proc muc::recv_negotiation {group res identities features extras} {
+    variable muc_compatible
+
     if {[cequal $res OK]} {
 	foreach f $features {
 	    set var [jlib::wrapper:getattr $f var]
 	    if {$var == $::NS(muc)} {
-		after idle \
-		    [list set [namespace current]::muc_compatible($group) 1]
+		set muc_compatible($group) 1
 		return
 	    }
 	}
     }
-    after idle [list set [namespace current]::muc_compatible($group) 0]
-}
-
-
-proc muc::wait_for_negotiation {group} {
-    variable muc_compatible
-    
-    if {[info exists muc_compatible($group)]} return
-
-    vwait [namespace current]::muc_compatible($group)
+    set muc_compatible($group) 0
 }
 
 
@@ -896,21 +930,21 @@ proc muc::join_group {connid group nick {password ""}} {
     global userstatus textstatus
     variable options
     variable timestamps
+    variable muc_password
 
     set chatid [chat::chatid $connid $group]
     set_our_groupchat_nick $chatid $nick
 
-    request_negotiation $connid $group
-
     chat::open_window $chatid groupchat
     update idletasks
 
-    wait_for_negotiation $group
+    request_negotiation $connid $group
 
     set x_subtags {}
     if {$password != ""} {
 	lappend x_subtags [jlib::wrapper:createtag password -chdata $password]
     }
+    set muc_password($chatid) $password
 
     set history_vars {}
     if {$options(history_maxchars) >= 0} {
@@ -943,8 +977,6 @@ proc muc::join_group {connid group nick {password ""}} {
 	lappend command -stat $textstatus
     }
     eval $command
-
-    client:presence $connid $group "" "" {}
 }
 
 proc muc::set_message_timestamp {chatid from type body x} {
@@ -953,7 +985,7 @@ proc muc::set_message_timestamp {chatid from type body x} {
     set timestamps($chatid) [clock seconds]
 }
 
-hook::add draw_message_hook [namespace current]::muc::set_message_timestamp 15
+hook::add draw_message_hook muc::set_message_timestamp 15
 
 proc muc::clear_message_timestamp {chatid} {
     variable timestamps
@@ -961,20 +993,24 @@ proc muc::clear_message_timestamp {chatid} {
     catch { unset timestamps($chatid) }
 }
 
-hook::add close_chat_post_hook [namespace current]::muc::clear_message_timestamp
+hook::add close_chat_post_hook muc::clear_message_timestamp
 
 ###############################################################################
 
 proc muc::invitation {chatid jid reason} {
     global invite_muc
 
-    message::send_msg $chatid -type normal \
+    set connid [chat::get_connid $chatid]
+    set to [chat::get_jid $chatid]
+
+    message::send_msg $to -type normal \
 	-xlist [list [jlib::wrapper:createtag x \
-	    -vars [list xmlns $::NS(gc_user)] \
+	    -vars [list xmlns $::NS(muc#user)] \
 	    -subtags [list [jlib::wrapper:createtag invite \
 		-vars [list to $jid] \
 		-subtags [list [jlib::wrapper:createtag reason \
-		    -chdata $reason]]]]]]
+		    -chdata $reason]]]]]] \
+	-connection $connid
 }
 
 
