@@ -1,4 +1,4 @@
-# $Id: draw_normal_message.tcl,v 1.8 2004/04/07 20:13:50 aleksey Exp $
+# $Id: draw_normal_message.tcl,v 1.10 2005/03/27 20:31:12 aleksey Exp $
 
 proc draw_normal_message {chatid from type body x} {
     if {[chat::is_our_jid $chatid $from]} {
@@ -7,8 +7,10 @@ proc draw_normal_message {chatid from type body x} {
 	set tag they
     }
 
+    set connid [chat::get_connid $chatid]
+
     set chatw [chat::chat_win $chatid]
-    set nick [chat::get_nick $from $type]
+    set nick [chat::get_nick $connid $from $type]
 
     $chatw insert end "<$nick>" [list $tag NICK-$nick] " "
 
@@ -18,7 +20,7 @@ proc draw_normal_message {chatid from type body x} {
     set cw [chat::winid $chatid]
     if {[cequal $type groupchat]} {
 	set myjid [chat::our_jid $chatid]
-	set mynick [chat::get_nick $myjid $type]
+	set mynick [chat::get_nick $connid $myjid $type]
 
 	lassign [check_message $mynick $body] mymessage mylist
 	if {$mymessage} {
@@ -44,4 +46,4 @@ proc draw_normal_message {chatid from type body x} {
 
     return stop
 }
-hook::add draw_message_hook [namespace current]::draw_normal_message 85
+hook::add draw_message_hook [namespace current]::draw_normal_message 87
