@@ -1,4 +1,4 @@
-# $Id: draw_signed.tcl,v 1.2 2003/07/16 14:50:08 aleksey Exp $
+# $Id: draw_signed.tcl,v 1.3 2005/04/25 23:56:56 aleksey Exp $
 
 proc draw_signed {chatid from type body x} {
     global messageicon
@@ -20,10 +20,11 @@ proc draw_signed {chatid from type body x} {
         }
 
         for {set i 0} {[winfo exists $chatw.signed$i]} {incr i} {}
+	set connid [chat::get_connid $chatid]
         catch {
             $chatw window create end \
                   -window [ssj::signed:Label $chatw.signed$i \
-                              [ssj::signed:input $from $chdata $body \
+                              [ssj::signed:input $connid $from $chdata $body \
                                   "Message body"]]
         }
     }
