@@ -1,15 +1,16 @@
-# $Id: me_command.tcl,v 1.5 2004/04/07 20:13:50 aleksey Exp $
+# $Id: me_command.tcl,v 1.7 2005/03/12 22:03:04 aleksey Exp $
 
 proc handle_me {chatid from type body x} {
-    if {[cequal [crange $body 0 3] "/me "]} {
+    if {[cequal [crange $body 0 3] "/me "] || [cequal $body "/me"]} {
 	if {[chat::is_our_jid $chatid $from]} {
 	    set tag me
 	} else {
 	    set tag they
 	}
 
+	set connid [chat::get_connid $chatid]
 	set chatw [chat::chat_win $chatid]
-	set nick [chat::get_nick $from $type]
+	set nick [chat::get_nick $connid $from $type]
 	set cw [chat::winid $chatid]
 
 	$chatw mark set MSGLEFT "end - 1 char"
