@@ -1,8 +1,23 @@
-# $Id: privacy.tcl,v 1.10 2004/07/15 21:56:10 aleksey Exp $
+# $Id: privacy.tcl,v 1.12 2005/06/22 02:56:27 aleksey Exp $
 
 namespace eval privacy {
     variable options
 
+    array set req_messages \
+	[list ignore    [::msgcat::mc "Requesting ignore list: %s"] \
+	      invisible [::msgcat::mc "Requesting invisible list: %s"] \
+	      visible   [::msgcat::mc "Requesting visible list: %s"]]
+
+    array set send_messages \
+	[list ignore    [::msgcat::mc "Sending ignore list: %s"] \
+	      invisible [::msgcat::mc "Sending invisible list: %s"] \
+	      visible   [::msgcat::mc "Sending visible list: %s"]]
+
+    array set edit_messages \
+	[list ignore    [::msgcat::mc "Edit ignore list"] \
+	      invisible [::msgcat::mc "Edit invisible list"] \
+	      visible   [::msgcat::mc "Edit visible list"]]
+
     custom::defgroup Privacy [::msgcat::mc "Blocking communication options."] -group Tkabber
 
     custom::defvar options(activate_at_startup) 1 \
@@ -712,11 +727,13 @@ proc privacy::edit_special_list {name args} {
 
 
 proc privacy::edit_special_list_dialog {connid name res child} {
+    variable req_messages
+    variable edit_messages
+
     if {![cequal $res OK]} {
 	if {[error_type_condition $child] != {cancel item-not-found}} {
 	    MessageDlg .privacy_list_err -aspect 50000 -icon error \
-		-message [format [::msgcat::mc "Requesting %s list: %s"] \
-			      $name [error_to_string $child]] \
+		-message [format $req_messages($name) [error_to_string $child]] \
 		-type user -buttons ok -default 0 -cancel 0
 	    return
 	}
@@ -732,7 +749,7 @@ proc privacy::edit_special_list_dialog {connid name res child} {
 	destroy $w
     }
 
-    Dialog $w -title [format [::msgcat::mc "Edit %s list"] $name] \
+    Dialog $w -title $edit_messages($name) \
 	-modal none -separator 1 -anchor e \
 	-default 0 -cancel 1
 
@@ -808,10 +825,11 @@ proc privacy::send_special_list {connid w name} {
 
 
 proc privacy::update_tkabber_lists {connid name res child} {
+    variable send_messages
+
     if {$res != "OK"} {
 	MessageDlg .privacy_list_err -aspect 50000 -icon error \
-	    -message [format [::msgcat::mc "Sending %s list: %s"] \
-			  $name [error_to_string $child]] \
+	    -message [format $send_messages($name) [error_to_string $child]] \
 	    -type user -buttons ok -default 0 -cancel 0
 	return
     }
