@@ -2,7 +2,7 @@
 # the next line restarts using the correct interpreter \
 exec wish "$0" -name tkabber "$@"
 
-# $Id: tkabber.tcl,v 1.219 2004/07/10 20:05:04 aleksey Exp $
+# $Id: tkabber.tcl,v 1.232 2005/07/31 21:09:36 aleksey Exp $
 
 set interface tk
 namespace eval ifacetk {}
@@ -54,7 +54,28 @@ package require BWidget
 
 tk appname tkabber
 
-set version 0.9.7
+if {[package provide starkit] != ""} {
+    set rootdir [file join $::starkit::topdir tkabber]
+} else {
+    set rootdir [file dirname [info script]]
+}
+
+proc get_snapshot {changelog} {
+    set snapshot ""
+    if {[catch { open $changelog } fd]} {
+	return $snapshot
+    }
+    while {[gets $fd line] >= 0} {
+	if {[regexp {(\d\d\d\d)-(\d\d)-(\d\d)} $line -> year month day]} {
+	    set snapshot "-$year$month$day"
+	    break
+	}
+    }
+    close $fd
+    return $snapshot
+}
+
+set version "0.9.8"
 set toolkit_version "Tcl/Tk [info patchlevel]"
 set debug_lvls {}
 
@@ -70,12 +91,6 @@ proc rescmd {id res ls} {
     puts "RESULT: $id $res $ls"
 }
 
-if {[package provide starkit] != ""} {
-    set rootdir [file join $::starkit::topdir tkabber]
-} else {
-    set rootdir [file dirname [info script]]
-}
-
 proc fullpath {args} {
     global rootdir
     return [eval file join [list $rootdir] $args]
@@ -104,25 +119,24 @@ proc load_source {args} {
 }
 
 proc quit {{status 0}} {
+    hook::run quit_hook
+    catch { bind $::ifacetk::mf <Destroy> {} }
     destroy .
-    terminate
+    exit $status
 }
 
-proc terminate {{status 0}} {
-    jlib::disconnect
-    hook::run quit_hook
+namespace eval ssj {}
+load_source hooks.tcl
 
+hook::add quit_hook logout 10
+hook::add quit_hook {
     foreach chan [file channels] {
         if {[string first sock $chan] != -1} {
             catch { close $chan } res
             debugmsg tkabber "closed $chan '$res'"
         }
     }
-    exit $status
-}
-
-namespace eval ssj {}
-load_source hooks.tcl
+} 100
 
 load_source default.tcl
 load_source ifacetk idefault.tcl
@@ -139,9 +153,14 @@ if {[file exists [file join ~ .tkabber config.tcl]]} {
     source [file join ~ .tkabber config.tcl]
 }
 
-if {$tcl_platform(platform) == "unix" && \
-	(![info exists load_default_xrdb] || $load_default_xrdb)} {
+if {![info exists load_default_xrdb] || $load_default_xrdb} {
     option readfile [fullpath ifacetk default.xrdb] startupFile
+
+    switch -- $tcl_platform(platform) {
+	unix {
+	    option readfile [fullpath ifacetk unix.xrdb] startupFile
+	}
+    }
 }
 
 set sysenc [encoding system]
@@ -184,6 +203,7 @@ foreach {opt val} $argv {
 	-chat {xmppmime::send_event [list chat $val]}
 	-message {xmppmime::send_event [list message $val]}
 	-conference {xmppmime::send_event [list groupchat $val]}
+	-splash {set show_splash_window $val}
     }
 }
 
@@ -220,7 +240,6 @@ load_source presence.tcl
 load_source iq.tcl
 load_source roster.tcl
 load_source itemedit.tcl
-load_source sound.tcl
 load_source messages.tcl
 load_source textundo dkflib.tcl
 load_source textundo textundo.tcl
@@ -242,7 +261,6 @@ load_source si.tcl
 load_source filetransfer.tcl
 load_source filters.tcl
 load_source privacy.tcl
-load_source avatars.tcl
 load_source gpgme.tcl
 
 load_source ifacetk bwidget_workarounds.tcl
