@@ -1,4 +1,4 @@
-# $Id: userinfo.tcl,v 1.52 2004/06/17 21:30:29 aleksey Exp $
+# $Id: userinfo.tcl,v 1.56 2005/04/25 23:56:56 aleksey Exp $
 
 
 namespace eval userinfo {
@@ -165,6 +165,7 @@ proc userinfo::open {jid args} {
     }
 
     toplevel $w -relief $::tk_relief -borderwidth $::tk_borderwidth
+    wm group $w .
     wm withdraw $w
     set title [format [::msgcat::mc "%s info"] $jid]
     wm title $w $title
@@ -312,10 +313,15 @@ proc userinfo::open {jid args} {
     }
     if {!$editable} {
 	pack_text_entry $jid $p 0 photo_extval [::msgcat::mc "URL:"]
-	label $p.photo -image $photo_img
-	grid $p.photo  -row 1 -column 0 -sticky we -columnspan 2
+	set sw [ScrolledWindow $p.sw]
+	grid $sw -row 1 -column 0 -sticky wens -columnspan 2 -pady 0.5m
 	grid rowconfig $p 1 -weight 1
-	#pack $p.photo -side top -expand yes -fill both
+	set sf [ScrollableFrame $p.sf]
+	$sw setwidget $sf
+	set l [label [$sf getframe].photo -image $photo_img]
+	pack $l
+	bindscroll $l $sf
+	bindscroll $sf $sf
     } else {
 	if {![info exists userinfo(photo_use,$jid)]} {
 	    set userinfo(photo_use,$jid) none
@@ -352,34 +358,22 @@ proc userinfo::open {jid args} {
 
     if {!$editable} {
 	$a.text configure -state disabled
-	set avatar_img [avatar::get_image [get_jid_of_user $connid $jid]]
-	if {$avatar_img != ""} {
-	    set avatar [$tab insert end avatar -text [::msgcat::mc "Avatar"]]
 
-	    set av [pack_frame $avatar.avatar [::msgcat::mc "Avatar"]]
-	    label $av.a -image $avatar_img
-	    pack $av.a -expand yes -fill both
-	}
-
-	set client [$tab insert end client -text [::msgcat::mc "Client"]]
-    
-	set c [pack_frame $client.client [::msgcat::mc "Client"]]
-	pack_entry $jid $c 0 clientname [::msgcat::mc "Client:"]
-	pack_entry $jid $c 1 clientversion [::msgcat::mc "Version:"]
-
-	set l [pack_frame $client.last [::msgcat::mc "Last Activity or Uptime"]]
-	pack_entry $jid $l 0 lastseconds [::msgcat::mc "Interval:"]
-	pack_entry $jid $l 1 lastdesc [::msgcat::mc "Description:"]
-
-	set o [pack_frame $client.computer [::msgcat::mc "Computer"]]
-	pack_entry $jid $o 0 os [::msgcat::mc "OS:"]
-	pack_entry $jid $o 1 time [::msgcat::mc "Time:"]
-	pack_entry $jid $o 2 tz   [::msgcat::mc "Time Zone:"]
-	pack_entry $jid $o 3 utc  [::msgcat::mc "UTC:"]
+	hook::run userinfo_hook $tab $connid $jid $editable
 
 	global presence
-	if {[llength [set jids [array names presence \
-				    signed,$connid,[node_and_server_from_jid $jid]/*]]] > 0} {
+	set bare_jid [node_and_server_from_jid $jid]
+	set chatid [chat::chatid $connid $bare_jid]
+	if {[chat::is_groupchat $chatid]} {
+	    if {[info exists presence(signed,$connid,$jid)]} {
+		set jids [list signed,$connid,$jid]
+	    } else {
+		set jids [list]
+	    }
+	} else {
+	    set jids [array names presence signed,$connid,$bare_jid/*]
+	}
+	if {[llength $jids] > 0} {
 	    set presenceinfo [$tab insert end presenceinfo -text [::msgcat::mc "Presence"]]
 	    set i 0
 	    foreach j $jids {
@@ -449,11 +443,6 @@ proc userinfo::open {jid args} {
 		}
 	    }
 	}
-
-	# FIX -to ...
-	request_iq version $connid $jid
-	request_iq time $connid $jid
-	request_iq last $connid $jid
     }
 
 
@@ -470,10 +459,36 @@ proc userinfo::open {jid args} {
     bind $w <Control-Next> [list ifacetk::tab_move $tab 1]
 
     $tab raise $top_page
+    update idletasks
 
     wm deiconify $w
 }
 
+proc userinfo::client_page {tab connid jid editable} {
+    set client [$tab insert end client -text [::msgcat::mc "Client"]]
+
+    set c [pack_frame $client.client [::msgcat::mc "Client"]]
+    pack_entry $jid $c 0 clientname [::msgcat::mc "Client:"]
+    pack_entry $jid $c 1 clientversion [::msgcat::mc "Version:"]
+
+    set l [pack_frame $client.last [::msgcat::mc "Last Activity or Uptime"]]
+    pack_entry $jid $l 0 lastseconds [::msgcat::mc "Interval:"]
+    pack_entry $jid $l 1 lastdesc [::msgcat::mc "Description:"]
+
+    set o [pack_frame $client.computer [::msgcat::mc "Computer"]]
+    pack_entry $jid $o 0 os [::msgcat::mc "OS:"]
+    pack_entry $jid $o 1 time [::msgcat::mc "Time:"]
+    pack_entry $jid $o 2 tz   [::msgcat::mc "Time Zone:"]
+    pack_entry $jid $o 3 utc  [::msgcat::mc "UTC:"]
+
+    # FIX -to ...
+    request_iq version $connid $jid
+    request_iq time $connid $jid
+    request_iq last $connid $jid
+}
+
+hook::add userinfo_hook [namespace current]::userinfo::client_page
+
 proc userinfo::enable_active_photo {p jid} {
     switch -- $userinfo::userinfo(photo_use,$jid) {
 	url {
